using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MimeKit;
using communicate_rental_backend_dev.Repositories.BlobStorage;

namespace communicate_rental_backend_dev.BusinessLogic.Services
{
    public class AllocationService : IAllocationService
    {
        private readonly IBookingRepository bookingRepository;
        private readonly IBlobBookingRepository blobBookingRepository;
        private readonly IBookingMapper bookingMapper;
        private readonly IEmailService emailService;
        private readonly IGraphService graphService;
        private readonly IUserRepository userRepository;

        public AllocationService(
        IBookingRepository bookingRepository,
        IBookingMapper bookingMapper,
        IEmailService emailService,
        IGraphService graphService,
        IUserRepository userRepository,
        IBlobBookingRepository blobBookingRepository)
        {
            this.bookingRepository = bookingRepository;
            this.bookingMapper = bookingMapper;
            this.emailService = emailService;
            this.graphService = graphService;
            this.userRepository = userRepository;
            this.blobBookingRepository = blobBookingRepository;
        }

        /// <summary>
        /// Is triggered by function app for particular rentalObject
        /// Gets all bookingrequests that starts in the allocationPeriod from booking database
        /// Takes booking with highestpriority and allocates this booking as long as there is any bookings in the list of bookings
        /// Each time an booking is allocated: 
        /// userPoints is decreased (by 2 if holiday, else by 1 point)
        /// Gets email from microsoft graph and sends confirmationEmail to user
        /// Checks for conflicting booking and deletes this booking from table TODO: upload to blob before object is deleted
        /// </summary>
        /// <param name="allocationPeriod">Allocationperiod that should be allocated</param>
        /// <returns></returns>
        public async Task Allocate(AllocationPeriod allocationPeriod)
        {
            // Trig this action on day after applicationDeadLine
            
            var bookingRequests = await GetBookingRequests(allocationPeriod);
            if (bookingRequests == null)
            {
                throw new ArgumentNullException("Couldnt find any bookings in period: " + allocationPeriod.Id);
            }
            var bookingRequestsList = bookingRequests.ToList();
            int counter = 0;
        
            while (bookingRequestsList.Count() != 0)
            {
                Booking allocatedBooking = await GetBookingWithHighestPriority(bookingRequestsList); 
                allocatedBooking.IsConfirmed = true;

                allocatedBooking.IsInHoliday = await bookingRepository.IsBookingInHoliday(allocatedBooking.StartDate, allocatedBooking.EndDate);

                User user = await userRepository.GetById(allocatedBooking.UserId);

                if (user.Points > 0) { user.Points --; }

                if (allocatedBooking.IsInHoliday && user.Points > 0) { user.Points --; }  

                // get email for user from microsoft graph
                MailboxAddress email = await graphService.GetEmailAsync(user.UserId);
                emailService.SendBookingConfirmation(email, allocatedBooking);

                await userRepository.Upload(user);
                await bookingRepository.Add(allocatedBooking);
                // Remove allocatedBooking from list of requests
                bookingRequestsList.Remove(allocatedBooking);
                counter ++;

                // Delete conflicting Bookings and upload to blob 
                foreach (var bookingRequest in bookingRequestsList) 
                {
                    await DeleteBookingsWithConflicts(allocatedBooking, bookingRequest);
                }  
            }
            Console.WriteLine("Period contains " + bookingRequests.Count() + "requests, " + counter +
            " bookings allocated on date:" + DateTime.Today.ToShortDateString()); 
        }


        // Get bookingsrequests for allocationperiod for rentalObject
        public async Task<IEnumerable<Booking>> GetBookingRequests(AllocationPeriod allocationPeriod)
        {
            return await bookingRepository.GetByRentalObjectInAllocationPeriod(allocationPeriod);
        }

        /// <summary>
        /// Compares the userPoints of the booking-owners of two different bookings
        /// If the userpoints equals each other -> randomize result
        /// </summary>
        /// <param name="bookings"></param>
        /// <returns></returns>
        public async Task<Booking> GetBookingWithHighestPriority(List<Booking> bookings)
        {
            // stores booking owned by booking with highest points -> initliazed to be first
            Booking tmpBooking = bookings.First();    

            // tmp that stores highest points -> initilized to be first booking                             
            int highestPoints = await GetUserPointsFromBooking(bookings.First().UserId);  
            
            // points for booking that will be compared to prioritized booking  
            int currentPoints;                                                          

            foreach (var booking in bookings)
            {
                currentPoints = await GetUserPointsFromBooking(booking.UserId);

                if (currentPoints > highestPoints)
                {
                    highestPoints = currentPoints;                   // sets new prioritized booking
                    tmpBooking = booking; 
                }
                if (currentPoints == highestPoints)
                {
                    // Randomize the result, currentPoints has index 0, highest points index 1
                    var rnd = new Random();
                    int index = rnd.Next(1);          

                    if (index == 0)                    
                    {
                        tmpBooking = booking;
                    }
                }
            }
            return tmpBooking;
        }

        /// <summary>
        /// Gets userpoints from a booking
        /// </summary>
        /// <param name="userId">if of the user</param>
        /// <returns></returns>
        public async Task<int> GetUserPointsFromBooking (string userId)
        {
            User user = await userRepository.GetById(userId);
            return user.Points;
        }
        
        /// <summary>
        /// Checks if a booking conflicts with the booking with highest priority
        /// If the booking have matching days (except start and enddate), the booking is deleted
        /// and an email is sent to user
        /// </summary>
        /// <param name="allocatedBooking">The booking that has been allocated</param>
        /// <param name="request">the current booking - bookingrequest (not allocated)</param>
        /// <returns></returns>
        public async Task DeleteBookingsWithConflicts(Booking allocatedBooking, Booking request)
        {
            var match = CheckForConflictingBookings(allocatedBooking, request);
            if (match) 
            {
                await blobBookingRepository.UploadAsyncDeletedRequests(request);
                await bookingRepository.Delete(request.EnglishTitle, request.Id); 

                var email = await graphService.GetEmailAsync(allocatedBooking.UserId);
                emailService.SendBookingCancellation(email, request);
            }
        }

        /// <summary>
        /// Loops trough dates of bookings to check for conflicting days
        /// </summary>
        /// <param name="allocatedBooking"></param>
        /// <param name="request"></param>
        /// <returns>boolean</returns>
        public bool CheckForConflictingBookings(Booking allocatedBooking, Booking request)
        {
            // Loops from trough dates of alllocated booking
            for (DateTime date = allocatedBooking.StartDate.AddDays(1).Date; date.Date < allocatedBooking.EndDate.Date; date = date.AddDays(1))
            {
                // loops trough dates of requests and look for matching dates
                for (DateTime d = request.StartDate.Date; d.Date <= request.EndDate.Date; d = d.AddDays(1))
                {
                    if (date.Date == d.Date) 
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}
