using System.Threading;
using System;
using communicate_rental_backend_dev.Configurations;
using communicate_rental_backend_dev.Models.Database;
using MimeKit;
using System.Globalization;
using System.Collections.Generic;
using System.Threading.Tasks;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Common;
using System.Linq;

namespace communicate_rental_backend_dev.BusinessLogic.Services
{
    public class EmailService : IEmailService
    {
        private readonly IEmailConfig emailConfig;
        private readonly IRentalObjectRepository rentalObjectRepository;
        private readonly IGraphService graphService;
        private readonly IMapperUtility mapperUtility;
        
        public EmailService(IEmailConfig emailConfig,
        IRentalObjectRepository rentalObjectRepository,
        IGraphService graphService,
        IMapperUtility mapperUtility)
	    {
		    this.emailConfig = emailConfig;
            this.rentalObjectRepository = rentalObjectRepository;
            this.graphService = graphService;
            this.mapperUtility = mapperUtility;
	    }

        /// <summary>
        /// Sends booking confirmation to user that requested the booking
        /// Is called by AllocationService
        /// </summary>
        /// <param name="toUserEmail">MailboxAddress of user from microsoft graph</param>
        /// <param name="booking">The booking that is confirmed in allocationService</param>
        /// <returns></returns>
        public bool SendBookingConfirmation(MailboxAddress toUserEmail, Booking booking)
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo("no-NO");
            var message = new MimeMessage();
            message.To.Add(toUserEmail);
            message.From.Add(new MailboxAddress("Communicate utleiesider", emailConfig.SmtpUsername));

            message.Subject = "Din booking fra " + booking.StartDate.ToLocalTime().ToShortDateString() + " - " + 
            booking.EndDate.ToLocalTime().ToShortDateString()  + " er tildelt";

            message.Body = new TextPart(MimeKit.Text.TextFormat.Html)
            {
                Text = string.Format (@"<p>Du har blitt tidelt din booking for " + booking.Title + " fra " + 
                booking.StartDate.Date.ToLocalTime().ToShortDateString() + 
                " til " + booking.EndDate.Date.ToLocalTime().ToShortDateString() + ".<br><br>" +
                "Med vennlig hilsen<br>" +
                "Communicate utleietjeneste<br></p>")
            };
            try
            {
                SendEmail(message);
            }
            catch (System.Exception)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Tells a user that the booking request got cancelled
        /// Is sent when a booking is conflicting with an allocated booking (invoked by allocationService)
        /// </summary>
        /// <param name="toUserEmail">MailboxAddress of user from graph</param>
        /// <param name="booking">The booking that is cancelled</param>
        /// <returns></returns>
        public bool SendBookingCancellation(MailboxAddress toUserEmail, Booking booking) 
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo("no-NO");
            var message = new MimeMessage();
            message.To.Add(toUserEmail);
            message.From.Add(new MailboxAddress("Communicate utleisider", emailConfig.SmtpUsername));

            if(!booking.IsConfirmed)
            {
                message.Subject = "Din bookingforespørsel for " + booking.StartDate.ToLocalTime().ToShortDateString() + " - " + 
                booking.EndDate.ToLocalTime().ToShortDateString()  + " er kansellert";
            }

            if (booking.IsConfirmed)
            {
                message.Subject = "Din booking for " + booking.StartDate.ToLocalTime().ToShortDateString() + " - " + 
                booking.EndDate.ToLocalTime().ToShortDateString()  + " er kansellert";
            }

            message.Body = new TextPart(MimeKit.Text.TextFormat.Html)
            {
                Text = string.Format (@"<p>Din booking for " + booking.Title + " fra " + booking.StartDate.Date.ToLocalTime().ToShortDateString() + 
                " til " + booking.EndDate.Date.ToLocalTime().ToShortDateString() + " har blitt kansellert.<br><br>" +
                "Med vennlig hilsen<br>" +
                "Communicate utleietjeneste<br></p>")
            };
            try
            {
                SendEmail(message);
            }
            catch (System.Exception)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Sends a reminder to all users 15 days before the allocationDeadline
        /// Invoked by function app (see FunctionAppController)
        /// </summary>
        /// <param name="toUserEmail">MailboxAddress of the user</param>
        /// <param name="allocationPeriod">The allocationPeriod that ends</param>
        /// <returns></returns>
        public bool SendDeadlineReminder(MailboxAddress toUserEmail, AllocationPeriod allocationPeriod) 
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo("no-NO");
            var message = new MimeMessage();
           
            message.From.Add(new MailboxAddress("Communicate utleiesider", emailConfig.SmtpUsername));
            message.To.Add(toUserEmail);

            message.Subject = "Søknadsfrist for " + allocationPeriod.AllocationStart.Date.ToLocalTime().ToShortDateString() + 
            "-" + allocationPeriod.AllocationEnd.ToLocalTime().ToShortDateString() +
            " nærmer seg.";

            message.Body = new TextPart(MimeKit.Text.TextFormat.Html)
            {
                Text = string.Format (@"<p>Hei!<br><br>Minner om at søknadsfrist for " + allocationPeriod.AllocationStart.ToShortDateString() + 
                "-" + allocationPeriod.AllocationEnd.ToLocalTime().ToShortDateString() + " nærmer seg!<br><br>" +
                "Dersom du ønsker å leie et av våre feriesteder i dette tidsrommet anbefaler vi å sende søknad innen " +
                allocationPeriod.ApplicationDeadline.ToLocalTime().ToShortDateString() + ".<br><br>" +
                "Med vennlig hilsen<br>" +
                "Communicate utleietjeneste<br></p>")
            };
            try
            {
                SendEmail(message);
            }
            catch (System.Exception)
            {
                return false;
            }
            return true;
        }

        
        public async Task<bool> SendListOfBookingsToAccountant(MailboxAddress toUserEmail, List<Booking> bookings) 
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo("no-NO");
            var message = new MimeMessage();
           
            message.From.Add(new MailboxAddress("Communicate utleiesider", emailConfig.SmtpUsername));
            message.To.Add(toUserEmail);

            message.Subject = "Bookinger for den foregående måneden";

            var intro = string.Format (@"<p>Hei!<br><br>Her kommer en liste over bookinger den foregående måneden, husk å sjekke at alt stemmer før det videresendes til lønn.<br><br>");
            var goodbye = string.Format( "<br><br>Med vennlig hilsen<br>Communicate utleietjeneste<br></p>");
            var bookingString = "";
            
            foreach (var booking in bookings)
            {
                var user = await graphService.GetGraphUser(booking.UserId);
                var name = user.displayName;
                if (user.displayName == null)
                {
                    name = user.userPrincipalName;
                }

                var periodPrices = await rentalObjectRepository.GetPeriodPrice(booking.EnglishTitle);
                var periodPrice = mapperUtility.DeSerializeToArray<PeriodPrice>(periodPrices);
                var pris =  string.Concat(periodPrice[0].Price.Where(Char.IsDigit));


                string bookingData = "<b>" + name + "</b><br>" +
                "Lokasjon: " + booking.Title + "<br>" +
                "Startdato: " +  booking.StartDate.ToLocalTime().ToShortDateString() + "<br>" +
                "Sluttdato: " + booking.EndDate.ToLocalTime().ToShortDateString() + "<br>" +
                "Pris: " + pris + " kr<br>" + 
                "Helligdag: " + (booking.IsInHoliday ? "Ja<br>" : "Nei<br>") + "------------------------------<br>";

                bookingString += bookingData;
                
            }
            message.Body = new TextPart(MimeKit.Text.TextFormat.Html) 
            {
                Text = string.Format(intro + bookingString + goodbye)
            };

            try
            {
                SendEmail(message);
            }
            catch (System.Exception)
            {
                return false;
            }
            return true;
        }

        public bool SendChecklistReminder(MailboxAddress toUserEmail,  string bookingTitle) 
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo("no-NO");
            var message = new MimeMessage();
           
            message.From.Add(new MailboxAddress("Communicate utleiesider", emailConfig.SmtpUsername));
            message.To.Add(toUserEmail);

            message.Subject = "Husk og levere skal sjekkliste for ditt opphold på " + bookingTitle;

            message.Body = new TextPart(MimeKit.Text.TextFormat.Html)
            {
                Text = string.Format (@"<p>Hei!<br><br>Minner om at det er obligatorisk og levere sjekkeliste etter endt opphold på et av
                våre utleiesteder. Gå inn i oversikten over dine bookinger, og klikk på <åpne sjekkliste> eller lenken under.<br>" + 
                "https://communicate-rental-dev.azurewebsites.net/bookings<br><br>"  +
                "Med vennlig hilsen<br>" +
                "Communicate utleietjeneste<br></p>")
            };
            try
            {
                SendEmail(message);
            }
            catch (System.Exception)
            {
                return false;
            }
            return true;
        }


        /// <summary>
        /// Sets up the SmtpClient and sends message
        /// </summary>
        /// <param name="message">Mimemessage - email class</param>
        /// <returns></returns>
        public bool SendEmail(MimeMessage message) 
        {
            try
            {
                using (var emailClient = new MailKit.Net.Smtp.SmtpClient())
                {
                    emailClient.Connect(emailConfig.SmtpServer, emailConfig.SmtpPort, true);

                    emailClient.Authenticate(emailConfig.SmtpUsername, emailConfig.SmtpPassword);
                
                    emailClient.Send(message);
                    emailClient.Disconnect(true);
                }
            }
            catch (System.Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
            return true;
        }
    }
}