using System;
using System.Linq;
using System.Threading.Tasks;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using communicate_rental_backend_dev.Mappers;
using MimeKit;
using System.Collections.Generic;

namespace communicate_rental_backend_dev.BusinessLogic.Services
{
    public class FunctionAppService : IFunctionAppService
    {
        private readonly IAllocationService allocationService;
        private readonly IEmailService emailService;
        private readonly IGraphService graphService;
        private readonly IAllocationPeriodRepository allocationPeriodRepository;
        private readonly IUserRepository userRepository;
        private readonly IHolidayRepository holidayRepository;
        private readonly IBookingRepository bookingRepository;
        private readonly IBlobBookingRepository blobBookingRepository;
        private readonly IChecklistRepository checklistRepository;
        private readonly IBookingMapper bookingMapper;

        public FunctionAppService(
            IAllocationService allocationService,
            IEmailService emailService,
            IGraphService graphService,
            IAllocationPeriodRepository allocationPeriodRepository,
            IUserRepository userRepository,
            IHolidayRepository holidayRepository,
            IBookingRepository bookingRepository,
            IBlobBookingRepository blobBookingRepository,
            IChecklistRepository checklistRepository,
            IBookingMapper bookingMapper
        )
        {
            this.allocationService = allocationService;
            this.emailService = emailService;
            this.graphService = graphService;
            this.allocationPeriodRepository = allocationPeriodRepository;
            this.userRepository = userRepository;
            this.holidayRepository = holidayRepository;
            this.bookingRepository = bookingRepository;
            this.blobBookingRepository = blobBookingRepository;
            this.checklistRepository = checklistRepository;
            this.bookingMapper = bookingMapper;
        }

        /// <summary>
        /// Gets all users from database
        /// and gives additional point to those with hire anniversary
        /// </summary>
        /// <param name="today"></param>
        /// <returns></returns>
        public async Task UpdateUserPoints(DateTime today)
        {
            var users = await userRepository.GetUsersHiredToday(today);

            // Log error if no userss
            if (users == null || !users.Any())
            {
                System.Diagnostics.Trace.TraceError("No users found");
            }
            else 
            {
                foreach (var user in users)
                {
                    user.Points++;
                    try
                    {
                        await userRepository.Upload(user);
                    }
                    catch (System.Exception e)
                    {
                        System.Diagnostics.Trace.TraceError(e.ToString());
                    }
                }
            }
        }

        /// <summary>
        /// Gets all rental periods
        /// then sends out reminder email for periods starting in 14 days
        /// then allocates bookings for periods where deadline has passed
        /// </summary>
        /// <param name="today"></param>
        /// <returns></returns>
        public async Task AllocateRentals(DateTime today)
        {
            var allocationPeriods = await allocationPeriodRepository.GetAll();
            
            // Log error if no allocation periods
            if (allocationPeriods == null)
            {
                System.Diagnostics.Trace.TraceError("No allocation periods found");
            }
            else
            {
                // Loop over allocation periods 
                foreach (var allocationPeriod in allocationPeriods)
                {
                    // If today is 14 days before deadline - send booking reminder to all users
                    if (today.Date == allocationPeriod.ApplicationDeadline.AddDays(-14).Date)
                    {
                        await SendDeadlineReminders(allocationPeriod);
                    }

                    // If deadline was yesterday - allocate
                    if (today.AddDays(-1).Date == allocationPeriod.ApplicationDeadline.Date)
                    {
                        await Allocate(allocationPeriod);
                    }
                }
            }
        }

        /// <summary>
        /// Sends bookingreminder to all users, throws system exeption if it doesnt work
        /// </summary>
        /// <param name="allocationPeriod">current allocationPeriod</param>
        /// <returns></returns>
        public async Task SendDeadlineReminders(AllocationPeriod allocationPeriod)
        {
            var users = await userRepository.GetAll();
            if (users == null || !users.Any())
            {
                System.Diagnostics.Trace.TraceError("Could not find any users");
            }

            var tasks = users.Select(user => SendDeadlineReminder(user, allocationPeriod));
            Task.WaitAll(tasks as Task[]); 
        }

        private async Task SendDeadlineReminder(User user, AllocationPeriod allocationPeriod)
        {
            var email = await graphService.GetEmailAsync(user.UserId);
            emailService.SendDeadlineReminder(email, allocationPeriod);
        }

        /// <summary>
        /// allocates bookings in allocationperiod
        /// </summary>
        /// <param name="allocationPeriod">current allocationPeriod</param>
        /// <returns></returns>
        public async Task Allocate(AllocationPeriod allocationPeriod)
        {
            try
            {
                await allocationService.Allocate(allocationPeriod);
            }
            catch (System.Exception e)
            {
                System.Diagnostics.Trace.TraceError(e.ToString());
            }
        }

        /// <summary>
        /// Updates holidays in database (adds holidays for next year on 1.1), and deletes the holidays from last year
        /// </summary>
        /// <param name="today">date of today, used to </param>
        /// <returns></returns>
        public async Task UpdateHolidays(DateTime today)
        {
            // add new holidays to database
            await holidayRepository.AddPublicHolidaysInYearIfNotExisists(today.Year + 1);
                    
            // delete holidays that wont be in use
            var holidaysFromLastYear = await holidayRepository.GetByPredicate(today.Year - 1);
            if (holidaysFromLastYear != null && holidaysFromLastYear.Any())
            {
                foreach (var holiday in holidaysFromLastYear)
                {
                    await holidayRepository.Delete(holiday.Id);
                }
            } 
        }

        /// <summary>
        /// Updates the year of allocationPeriods
        /// </summary>
        /// <param name="today">datetime of today, triggered on 1.1</param>
        /// <returns></returns>
        public async Task UpdateAllocationPeriods(DateTime today)
        {
            try
            {
                // updates allocationPeriod table
                var allocationPeriodsToUpdate = await allocationPeriodRepository.GetByPredicate(today);
                if (allocationPeriodsToUpdate != null && allocationPeriodsToUpdate.Any())
                {
                    // Updates the year in each table
                    foreach (var period in allocationPeriodsToUpdate)
                    {
                        var updatedPeriod = allocationPeriodRepository.AddYear(period ,2);
                        await allocationPeriodRepository.Add(updatedPeriod);
                    }
                }
            }
            catch (System.Exception e)
            {
                System.Diagnostics.Trace.TraceError(e.ToString());
            }
        }

        /// <summary>
        /// If today is 32 days after enddate of booking and checklist is filled -> move to blob
        /// If today is 3 days or more after enddate booking and checklis int filled -> send reminder
        /// </summary>
        /// <param name="today">date of today (UTC)</param>
        /// <returns></returns>
        public async Task MoveBookingsToBlobOrSendChecklistReminder(DateTime today)
        {
            try
            {
                var bookings = await bookingRepository.GetBookingsPastEndDate(today);

                if (bookings != null || !bookings.Any())
                {
                    foreach (var booking in bookings)
                    {
                        var checklist = await checklistRepository.GetByBookingId(booking.Id);
                        
                        if (booking.EndDate.Date.AddDays(3) >= today.Date && checklist == null)
                        {
                            var email = await graphService.GetEmailAsync(booking.UserId);
                            emailService.SendChecklistReminder(email, booking.Title);
                        }

                        if (booking.EndDate.Date.AddDays(32) >= today.Date && checklist != null)
                        {
                            
                            await blobBookingRepository.UploadAsyncDeletedBooking(booking);
                            await bookingRepository.Delete(booking.Id, booking.EnglishTitle);
                        }
                    }
                }
            }
            catch (System.Exception e)
            {
                System.Diagnostics.Trace.TraceError(e.ToString());
            }
        }

        /// <summary>
        /// Converts list of bookings to string and sends email to accountant
        /// </summary>
        /// <param name="today"></param>
        /// <returns></returns>
        public async Task SendListOfBookingsForMonth(DateTime today)
        {
            try
            {
                if (today.Day == 1)
                {
                    var listOfBookings = new List<Booking>();
                    var listOfConfirmedBookings = await bookingRepository.GetConfirmedBookings();
                    if (listOfConfirmedBookings != null && listOfConfirmedBookings.Any())
                        foreach (var booking in listOfConfirmedBookings)
                        {
                            if (DateTime.Today.Date > booking.StartDate.Date
                            && booking.StartDate.Date >= new DateTime(today.Year, today.Month - 1, today.Day).Date)
                            {
                                listOfBookings.Add(booking);
                            }
                        }
                        listOfBookings.Sort((x, y) => x.UserId.CompareTo(y.UserId));
                        var email = new MailboxAddress("rental.communicate@gmail.com", "rental.communicate@gmail.com");
                        await emailService.SendListOfBookingsToAccountant(email, listOfBookings);
                }
                
            }
            catch (System.Exception e)
            {
                System.Diagnostics.Trace.TraceError(e.ToString());
            }
        }
            
    }

}