using MimeKit;
using System;
using System.Threading.Tasks;
using System.Net.Http;
using System.Collections.Generic;
using communicate_rental_backend_dev.Configurations;
using communicate_rental_backend_dev.Models.Common;
using System.Net.Http.Headers;
using System.Text.Json;

namespace communicate_rental_backend_dev.BusinessLogic.Services
{
    public class GraphService : IGraphService
    {
        
        private readonly HttpClient httpClient;
        private readonly IAzureAdConfig aadConfig;
        

        public GraphService(
            HttpClient httpClient, 
            IAzureAdConfig AzureAdConfig)
        {
            this.httpClient = httpClient;
            this.aadConfig = AzureAdConfig;
        }

        /// <summary>
        /// Calls microsoft graph to get email of user
        /// </summary>
        /// <param name="userId">The id of the user</param>
        /// <returns>MailboxAddress of the user</returns>
        public async Task<MailboxAddress> GetEmailAsync(string userId)
        {
            var graphUser = await GetGraphUser(userId); 

            if (graphUser == null)
            {
                return null; 
            }

            if (graphUser.mail != null) {
                return new MailboxAddress(graphUser.mail, graphUser.mail);
            }
        
            return new MailboxAddress(graphUser.userPrincipalName, graphUser.userPrincipalName);
        }

        /// <summary>
        /// gets graphUser by id, and calls graph to get the name of a user
        /// </summary>
        /// <param name="userId">id in microsoft graph</param>
        /// <returns>string that contains name of the user</returns>
        public async Task<string> GetUserName(string userId)
        {
            var graphUser = await GetGraphUser(userId);
            if (graphUser.displayName == null)
            {
                return null;
            }
            return graphUser.displayName;
        }

        public async Task<string> GetUserId(string email)
        {
            var graphUser = await GetGraphUser(email);
            if (graphUser?.id == null)
            {
                return null;
            }
            return graphUser.id;
        }


        /// <summary>
        /// Calls microsoft graph to get info of user
        /// </summary>
        /// <param name="userId">The id of the user</param>
        /// <returns>graphuser with email and username</returns>
        public async Task<GraphUser> GetGraphUser(string userId)
        {
            try
            {                
                var accessToken = await GetAccessTokenAsync();

                var requestMessage = new HttpRequestMessage(HttpMethod.Get, "https://graph.microsoft.com/v1.0/users/" + userId);
                requestMessage.Headers.Authorization = new AuthenticationHeaderValue("Bearer", accessToken);
                
                var response = await httpClient.SendAsync(requestMessage);
                if (!response.IsSuccessStatusCode) 
                {
                    return null;
                }
                var content = response.Content.ReadAsStringAsync().Result;

                var graphUser = JsonSerializer.Deserialize<GraphUser>(content);

                return graphUser;
            }
            catch (Exception e)
            {
                Console.WriteLine($"Error getting user: {e.Message}");
                return null;
            }
        }
        
        /// <summary>
        /// Calls microsoft graph to get oAuth accessToken for a user
        /// </summary>
        /// <returns>serialized accesstoken</returns>
        private async Task<string> GetAccessTokenAsync() 
        {
            var values = new Dictionary<string, string>
            {
                { "client_id", aadConfig.ClientId },
                { "scope", "https://graph.microsoft.com/.default" },
                { "client_secret", aadConfig.ClientSecret },
                { "grant_type", "client_credentials" }
            };

            string url = "https://login.microsoftonline.com/" + aadConfig.TenantId + "/oauth2/v2.0/token";
            var content = new FormUrlEncodedContent(values);
            var response = await httpClient.PostAsync(url, content);
            
            var accessToken = JsonSerializer.Deserialize<AccessToken>(response.Content.ReadAsStringAsync().Result);
            
            return accessToken.access_token; 
        }
    }    
}
