using Azure.Storage.Blobs;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace communicate_rental_backend_dev.Clients
{
    public class BlobStorageClient : IBlobStorageClient
    {
        private readonly BlobServiceClient client;

        public BlobStorageClient(string connectionString)
        {
            client = new BlobServiceClient(connectionString);
        }

        /* public async Task<List<IListBlobItem>> GetBlobAsync(string blobContainer)
        { 
            
            CloudBlobContainer container = _client.GetContainerReference(blobContainer);
            container.CreateIfNotExists
            var blockBlobReference = container.GetBlockBlobReference("");
            blockBlobReference
            
            return blobItems;
            //var containerClient = _client.GetBlobContainerClient("assets");
            //var blobClient = containerClient.GetBlobClient(name);
            var blobDownloadInfo =  await _client.get;
            return blobDownloadInfo;
        } */

        // Get method that only a list of url's (string) for all the blobs from a single container
        public async Task<IEnumerable<string>> GetAllURLs(string containerName) 
        {
            // Check that containerName string is not empty
            if (string.IsNullOrEmpty(containerName)) {
                // returns empty list 
                return Enumerable.Empty<string>();
            }

            BlobContainerClient container = client.GetBlobContainerClient(containerName);

            // Check that container exists
            if (!(await container.ExistsAsync())) 
            {
                return Enumerable.Empty<string>();
            }

            // Get all urls in container
            var urls = new List<string>();
            await foreach (var blobItem in container.GetBlobsAsync())
            {
                // Perhaps add or remove a slash between the two components of the url
                urls.Add(container.Uri.AbsoluteUri + '/' + blobItem.Name);
            }
            return urls;
        }

        public async Task<string> GetContainerUri(string containerName)
        {
            // Check that containerName string is not empty
            if (string.IsNullOrEmpty(containerName)) {
                return "";
            }

            BlobContainerClient container = client.GetBlobContainerClient(containerName);

            // Check that container exists
            if (!(await container.ExistsAsync())) 
            {
                return "";
            }

            return container.Uri.AbsoluteUri;
        }

        /// <summary>
        /// Get Url of specified file
        /// </summary>
        /// <param name="containerName"></param>
        /// <param name="filename"></param>
        /// <returns>url in string format</returns>
        public async Task<string> GetURL(string containerName, string filename)
        {
            // Check that containerName string is not empty
            if (string.IsNullOrEmpty(containerName)) {
                // returns empty list 
                return null;
            }

            BlobContainerClient container = client.GetBlobContainerClient(containerName);

            // Check that container exists
            if (!(await container.ExistsAsync())) 
            {
                return null;
            }
            // Get all urls in container
            return container.Uri.AbsoluteUri + "/" + filename;
        }


        /// <summary>
        /// Uploads files to blob storage in azure
        /// </summary>
        /// <param name="fileStream">the file stream to upload</param>
        /// <param name="blobName">the name of the blob</param>
        /// <param name="containerName">the name of the container to store the blob in</param>
        /// <returns></returns>
        public async Task<bool> UploadBlobAsync(Stream fileStream, string blobName, string containerName)
        {
            // Check that necessary properties has content
            if (string.IsNullOrEmpty(containerName))
            {
                throw new ArgumentException("Container name cannot be null or empty.", nameof(containerName));
            }
            if (string.IsNullOrEmpty(blobName))
            {
                throw new ArgumentException("Blob name cannot be null or empty.", nameof(blobName));
            }

            // Get blob container client, create with neccessary access permissions if doesn't exist
            BlobContainerClient container = client.GetBlobContainerClient(containerName);
            await container.CreateIfNotExistsAsync(Azure.Storage.Blobs.Models.PublicAccessType.Blob);

            // Retrieve reference to blob
            BlobClient blobClient = container.GetBlobClient(blobName);

            // Upload file
            await blobClient.UploadAsync(fileStream, true);

            //await _blobContainer.SetPermissionsAsync(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Blob });
            return await blobClient.ExistsAsync();
        }

        /// <summary>
        /// Upload text to blob storage in azure
        /// </summary>
        /// <param name="content">the content of the file</param>
        /// <param name="blobName">the name of the blob</param>
        /// <param name="containerName">the name of the container to store the blob in</param>
        /// <returns></returns>
        public async Task<bool> UploadBlobAsync(string content, string blobName, string containerName)
        {
            // Check that necessary properties has content
            if (string.IsNullOrEmpty(containerName))
            {
                throw new ArgumentException("Cannot be null or empty.", nameof(containerName));
            }
            if (string.IsNullOrEmpty(blobName))
            {
                throw new ArgumentException("Blob name cannot be null or empty.", nameof(blobName));
            }

            // Get blob container client, create if doesn't exist
            BlobContainerClient container = client.GetBlobContainerClient(containerName);
            await container.CreateIfNotExistsAsync();

            // Retrieve reference to blob
            BlobClient blobClient = container.GetBlobClient(blobName);

            // Create stream from string and upload
            var stream = Encoding.UTF8.GetBytes(content);
            using(var ms = new MemoryStream(stream))
            {
                await blobClient.UploadAsync(ms);
            }

            return await blobClient.ExistsAsync();
        }

        /// <summary>
        /// Deletes the blobs and container
        /// </summary>
        /// <param name="containerName">name of container to be deleted</param>
        /// <returns>boolean</returns>
        public async Task<bool> DeleteContainer(string containerName)
        {
            // Check that containerName string is not empty
            if (string.IsNullOrEmpty(containerName)) {
                // returns empty list 
                return false;
            }

            BlobContainerClient container = client.GetBlobContainerClient(containerName);

            // Check that container exists
            if (!(await container.ExistsAsync())) 
            {
                return false;
            }
            // delete container
            if (container.DeleteAsync().IsCompleted) { return true; }
            return false;
        }

        /// <summary>
        /// deletes blobfiles in container
        /// </summary>
        /// <param name="containerName"></param>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public async Task<bool> DeleteBlob(string containerName, string fileName)
        {
            // Check that containerName string is not empty
            if (string.IsNullOrEmpty(containerName)) {
                // returns empty list 
                return false;
            }

            BlobContainerClient container = client.GetBlobContainerClient(containerName);

            // Check that container exists
            if (!(await container.ExistsAsync())) 
            {
                return false;
            }
            // delete container
            var response = await container.DeleteBlobAsync(fileName);

            if (response.Status >= 200 && response.Status < 300) 
            { 
                return true; 
            }
            return false;
        }
    }
}