﻿using Microsoft.Azure.Cosmos.Table;
using Microsoft.Azure.Cosmos.Table.Queryable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

// Generic class that doesn't care about what kinda data it takes in 
// Everything goes through this one to table storage 
namespace communicate_rental_backend_dev.Clients
{
    public class TableStorageClient<T> : ITableStorageClient<T> where T : TableEntity, new()
     {
        private readonly CloudStorageAccount storageAccount;
        private readonly CloudTableClient tableClient;
        private readonly CloudTable table;

        public TableStorageClient(string connectionString, string tableName)
        {
            storageAccount = CreateStorageAccountFromConnectionString(connectionString);
            tableClient = storageAccount.CreateCloudTableClient(new TableClientConfiguration());
            table = tableClient.GetTableReference(tableName);
            table.CreateIfNotExists();
        }

        public async Task<T> GetById(string partitionKey, string rowKey)
        {
            try
            {
                var operation = TableOperation.Retrieve<T>(partitionKey, rowKey);
                return (await table.ExecuteAsync(operation))?.Result as T;
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                return null;
            }
        }

        public async Task<IEnumerable<T>> GetAll()
        {
            var query = new TableQuery<T>();
            return await GetByQuery(query);
        }

        public async Task<IEnumerable<T>> GetByPredicate(Expression<Func<T, bool>> predicate)
        {
            var queryable = table.CreateQuery<T>().AsQueryable().Where(predicate);
            var query = queryable.AsTableQuery();

            return await GetByQuery(query);
        }

        public async Task Upsert(T entity, string rowKey, string partitionKey)
        {
            entity.RowKey = rowKey;
            entity.PartitionKey = partitionKey;

            var insertOrReplaceOperation = TableOperation.InsertOrMerge(entity);
            await table.ExecuteAsync(insertOrReplaceOperation);
        }

        public async Task Delete(string partitionKey, string rowKey)
        {
            var entity = await GetById(partitionKey, rowKey);
            if (entity == null)
            {
                return;
            }

            var deleteOperation = TableOperation.Delete(entity);
            await table.ExecuteAsync(deleteOperation);
        }

        private CloudStorageAccount CreateStorageAccountFromConnectionString(string storageConnectionString)
        {
            try
            {
                return CloudStorageAccount.Parse(storageConnectionString);
            }
            catch (FormatException)
            {
                Console.WriteLine("Invalid storage account information provided. Please confirm the AccountName and AccountKey are valid in appsettings.json - then restart the application.");
                throw;
            }
            catch (ArgumentException)
            {
                Console.WriteLine("Invalid storage account information provided. Please confirm the AccountName and AccountKey are valid in appsettings.json - then restart the application.");
                Console.ReadLine();
                throw;
            }
        }

        private async Task<IEnumerable<T>> GetByQuery(TableQuery<T> query)
        {
            var entities = new List<T>();

            TableContinuationToken continuationToken = null;
            do
            {
                var queryResult = await table.ExecuteQuerySegmentedAsync<T>(query, continuationToken);

                continuationToken = queryResult.ContinuationToken;
                entities.AddRange(queryResult.Results);

            } while (continuationToken != null);

            Console.WriteLine(entities);
            return entities;
        }
    }
}