using System;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.DataTransfer;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;

namespace communicate_rental_backend_dev.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class BookingsController : Controller
    {
        private readonly IAllocationPeriodRepository allocationPeriodRepository;
        private readonly IBookingRepository bookingRepository;
        private readonly IBookingMapper bookingMapper;
        private readonly IGraphService graphService;
        private readonly IEmailService emailService;
        private readonly IAllocationService allocationService;
        private readonly IUserRepository userRepository;

        public BookingsController(
            IAllocationPeriodRepository allocationPeriodRepository,
            IBookingRepository bookingRepository, 
            IBookingMapper bookingMapper,
            IEmailService emailService,
            IGraphService graphService,
            IAllocationService allocationService,
            IUserRepository userRepository)
        {
            this.allocationPeriodRepository = allocationPeriodRepository;
            this.bookingRepository = bookingRepository;
            this.bookingMapper = bookingMapper;
            this.emailService = emailService;
            this.graphService = graphService;
            this.allocationService = allocationService;
            this.userRepository = userRepository;
        }

        /// <summary>
        /// Get all bookings from database
        /// </summary>
        /// <returns>IEnumerable list of all bookings</returns>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<BookingDTO>>> GetBookings()
        {
            var bookings = await bookingRepository.GetAll();
            if (bookings == null || !bookings.Any())
            {
                return NotFound();
            }

            var tasks = bookings.Select(booking => GetBookingDTO(booking));
            return await Task.WhenAll(tasks);
        }

        /// <summary>
        /// Get the bookings requested by a user
        /// </summary>
        /// <param name="userId">Id of the user</param>
        /// <returns>IEnumerable list of bookings for the user</returns>
        [HttpGet]
        [Route("users/{userId}")]
        public async Task<ActionResult<IEnumerable<BookingDTO>>> GetByUser([FromRoute] string userId)
        {
            var bookings = await bookingRepository.GetByUserId(userId);
            if (bookings == null || !bookings.Any())
            {
                return NotFound();
            }
            
            return bookings.Select(booking => bookingMapper.Map(booking)).ToList();
        }

        /// <summary>
        /// Get the bookings for a rentalObject with startdate after today
        /// </summary>
        /// <param name="englishTitle">id of the booking</param>
        /// <returns>IEnumerable list of bookings</returns>
        [HttpGet]
        [Route("locations/{englishTitle}")]
        public async Task<ActionResult<IEnumerable<BookingDTO>>> GetByRentalObject([FromRoute] string englishTitle)
        {
            var bookings = await bookingRepository.GetByRentalObject(englishTitle);
            if (bookings == null || !bookings.Any())
            {
                return NotFound();
            }

            var tasks = bookings.Select(booking => GetBookingDTO(booking));
            return await Task.WhenAll(tasks);
        }

        /// <summary>
        /// Adds a bookingsrequest/booking to db if there is no conflicting bookings, and booking is after deadline
        /// </summary>
        /// <param name="bookingDRO">Data Recieving Object of user type</param>
        /// <returns>Statuscode of operation</returns>
        [HttpPost]
        public async Task<ActionResult> Add([FromBody]BookingDRO bookingDRO) 
        {
            if (bookingDRO.StartDate.Date > DateTime.Today.Date.AddYears(1))
            {
                return Conflict("Kan ikke legge til booking mer enn ett år frem i tid");
            } 
            // Map booking to database object
            Booking booking = bookingMapper.Map(bookingDRO); 

            // Get current allocationPeriod
            var currentPeriods = await allocationPeriodRepository.GetCurrent(DateTime.Today.ToUniversalTime(), booking.EnglishTitle);
            var currentPeriod = currentPeriods.FirstOrDefault();
        
            // Check if booking should immediately be confirmed/assigned or not
            bool shouldAssign = currentPeriod == null ? 
                true : booking.StartDate >= currentPeriod.AllocationStart &&
                        booking.StartDate < currentPeriod.AllocationEnd;

            // If immediately confirmed: save to database and send confirmation email
            if (shouldAssign) 
            {
                // Check that there is no bookings on the requsted dates
                var conflictingBookingsOnLocation = await bookingRepository.GetAllocatedBookingsOnLocation(booking.EnglishTitle);
                if (conflictingBookingsOnLocation != null && conflictingBookingsOnLocation.Any())
                {
                    foreach (var locationBooking in conflictingBookingsOnLocation)
                    {
                        var matchOnLocation = allocationService.CheckForConflictingBookings(locationBooking, booking);
                        if (matchOnLocation)
                        {
                            return Conflict("det ligger allerede en booking for lokasjonen i denne tidsperioden");
                        }
                    }
                }
                
                // check that the user havent booked on several rentalobjects on the same daterange
                var conflictingBookingOnUser = await bookingRepository.GetAllocatedBookingsForUser(booking.UserId);
                if (conflictingBookingOnUser != null && conflictingBookingOnUser.Any())
                {
                    foreach (var userBooking in conflictingBookingOnUser) 
                    {
                        var matchOnUser = allocationService.CheckForConflictingBookings(userBooking, booking);
                        if (matchOnUser)
                        {
                            return Conflict("du har allerede en booking i denne tidsperioden");
                        }
                    }
                }
                
                booking.IsConfirmed = true; 
                await bookingRepository.Add(booking);

                // Send email confirmation to user
                var email = await graphService.GetEmailAsync(booking.UserId);
                if (email != null)
                {
                    emailService.SendBookingConfirmation(email, booking);
                }

                var user = await userRepository.GetById(booking.UserId);
                if (user.Points > 0) { user.Points --; }

                if (user.Points > 0) 
                {
                    // Check if booking is in holiday, remove extra point if it is
                    booking.IsInHoliday = await bookingRepository.IsBookingInHoliday(booking.StartDate, booking.EndDate);
                    if (booking.IsInHoliday) { user.Points --;}
                }
                
                
                await userRepository.Upload(user);
                
                return Created(booking.Id, true);
            } 
            // If not immediately confirmed: save to database for later allocation
            else {
                await bookingRepository.Add(booking);
                return Accepted(booking.Id, true);
            }
            
        }

        [HttpPut]
        public async Task<IActionResult> AddComment([FromBody]BookingDRO bookingWithCommentDRO)
        {
            var bookingWithComment = bookingMapper.MapUpdate(bookingWithCommentDRO);
            await bookingRepository.Add(bookingWithComment);
            return Accepted(bookingWithComment.Id, true);
        }
         
        /// <summary>
        /// Deletes booking and send email to user
        /// </summary>
        /// <param name="englishTitle"></param>
        /// <param name="bookingId"></param>
        /// <returns></returns>
        [HttpDelete]
        [Route("locations/{englishTitle}/{bookingId}")]
        public async Task<ActionResult<bool>> Delete([FromRoute]string englishTitle, string bookingId) 
        {
            var booking = await bookingRepository.GetById(englishTitle, bookingId);
            if (booking != null)
            {
                if (await bookingRepository.Delete(englishTitle, bookingId)) 
                {
                    var email = await graphService.GetEmailAsync(booking.UserId);
                    emailService.SendBookingCancellation(email, booking);
                    return Ok(true);
                }
            }
            return StatusCode(500, false);
        }

        private async Task<BookingDTO> GetBookingDTO(Booking booking)
        {
            var name = await graphService.GetUserName(booking.UserId); 
            return bookingMapper.Map(booking, name);
        }
    }
} 