using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using communicate_rental_backend_dev.Models.Common;

namespace communicate_rental_backend_dev.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class ChecklistsController : Controller
    {
        private readonly IChecklistRepository checklistRepository;
        private readonly IChecklistMapper checklistMapper;
        private readonly IRentalObjectRepository rentalObjectRepository;
        private readonly IGraphService graphService;

        public ChecklistsController(
            IChecklistRepository checklistRepository,
            IChecklistMapper checklistMapper,
            IGraphService graphService,
            IRentalObjectRepository rentalObjectRepository)
        {
            this.checklistRepository = checklistRepository;
            this.checklistMapper = checklistMapper;
            this.graphService = graphService;
            this.rentalObjectRepository = rentalObjectRepository;
        }

        /// <summary>
        /// Get all checklists 
        /// </summary>
        /// <returns>List of checklists</returns>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<ChecklistDTO>>> GetChecklists()
        {
            var checklists = await checklistRepository.GetAll();
            if (checklists == null || !checklists.Any())
            {
                return NotFound();
            }

            var tasks = checklists.Select(list => GetChecklistDTO(list));
            return await Task.WhenAll(tasks);
        }

        /// <summary>
        /// Get checklist from bookingId, if checklist doesnt exist -> empty checklist
        /// </summary>
        /// <param name="englishTitle"> djgjdsfjgjg</param>
        /// <param name="bookingId"></param>
        /// <returns></returns>
        [HttpGet("{englishTitle}/{bookingId}")]
        public async Task<ActionResult<ChecklistDTO>> Get([FromRoute] string englishTitle, string bookingId)
        {
            var filledChecklists = await checklistRepository.GetByBookingId(bookingId);
            if (filledChecklists == null || !filledChecklists.Any())
            {
                var rentalObjectChecklist = await rentalObjectRepository.GetChecklist(englishTitle);
                if (rentalObjectChecklist == null)
                {
                    return NotFound();
                }

                return checklistMapper.Map(rentalObjectChecklist);
            }
            var filledChecklist = filledChecklists.First();
            return checklistMapper.Map(filledChecklist, "");
        }

        /// <summary>
        /// Adds new checklist to database
        /// </summary>
        /// <param name="checklistDRO"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<ActionResult<bool>> UploadChecklist([FromBody] ChecklistDRO checklistDRO)
        {
            Checklist checklist = checklistMapper.Map(checklistDRO);
            // if it exists a checklist for a booking -> update with same id
            var existingChecklist = await checklistRepository.GetByBookingId(checklistDRO.BookingId);
            if (existingChecklist!=null && existingChecklist.Any())
            {
                checklist.Id = existingChecklist.First().Id;
            }
            await checklistRepository.Add(checklist);
            return Created(checklist.Id, true);
        }

        /// <summary>
        /// Deletes checklist with id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{id}")]
        public async Task<IActionResult> Delete([FromRoute] string id)
        {
            await checklistRepository.Delete(id);

            return StatusCode(200, "Checklist with id: " + id + " was deleted");
        }

        private async Task<ChecklistDTO> GetChecklistDTO(Checklist list)
        {
            var name = await graphService.GetUserName(list.UserId);
            return checklistMapper.Map(list, name);
        }
    }
}