using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Linq;
using System.IO;

namespace communicate_rental_backend_dev.Controllers 
{
    [ApiController]
    [Route("[controller]")]
    public class RentalObjectController : Controller 
    {
        private readonly IAllocationPeriodMapper allocationPeriodMapper;
        private readonly IAllocationPeriodRepository allocationPeriodRepository;
        private readonly IImageRepository imageRepository;
        private readonly IRentalObjectMapper rentalObjectMapper;
        private readonly IRentalObjectRepository rentalObjectRepository;

        // What happens here?
        //  Send calls to itemRepository
        //  which calls clients that access the data
        //  and returns it here

        public RentalObjectController(
            IAllocationPeriodMapper allocationPeriodMapper,
            IAllocationPeriodRepository allocationPeriodRepository,
            IImageRepository imageRepository,
            IRentalObjectMapper rentalObjectMapper,
            IRentalObjectRepository rentalObjectRepository)
        {
            this.rentalObjectRepository = rentalObjectRepository;
            this.imageRepository = imageRepository;
            this.allocationPeriodRepository = allocationPeriodRepository;
            this.rentalObjectMapper = rentalObjectMapper;
            this.allocationPeriodMapper = allocationPeriodMapper;
        }

        /// <summary>
        /// Call repository -> db to get all rentalObjects
        /// Calls imageRepo to get all imageURLs for blob storage
        /// Maps imageUrls to array of strings in rentalObjectDTO
        /// </summary>
        /// <returns>Data Transfer object of type RentalObject</returns>
        [HttpGet]
        public async Task<IEnumerable<RentalObjectDTO>> GetRentalObjects() 
        {
            var rentalObjects = await rentalObjectRepository.GetAll();

            if (rentalObjects == null)
            {
                throw new ArgumentNullException("Could not find any rental objects");
            }

            var rentalDTOs = new List<RentalObjectDTO>(); 

            foreach (var rentalObject in rentalObjects)
            {   
                var imageUrls = await imageRepository.GetAllURLs(rentalObject.EnglishTitle);
                rentalDTOs.Add(rentalObjectMapper.Map(rentalObject, imageUrls)); 
            }
            Console.WriteLine(rentalDTOs); 
            return rentalDTOs;
        }


        [HttpGet]
        [Route("ids")]
        public async Task<IEnumerable<RentalObjectID>> GetRentalObjectIds() 
        {
            var ids = await rentalObjectRepository.GetIds();
            if (ids == null)
            {
                throw new ArgumentNullException("Couldnt find Ids");
            }
            return ids;
        }

        /// <summary>
        /// Call rentalObject repo to get by Id
        /// Checks if id exists in db
        /// Call imagerepo to get urls, mapper handles null value for urls
        /// maps rentalobject to DTO
        /// </summary>
        /// <param name="Id">englishtitle of the rentalObject</param>
        /// <returns>rentalObject data Transfer</returns>
        [HttpGet]
        [Route("{id}")]
        public async Task<RentalObjectDTO> Get([FromRoute]string Id) 
        {
            var rentalObject = await rentalObjectRepository.GetById(Id);
            if (rentalObject == null)
            {
                throw new ArgumentNullException("Couldnt find rentalObject with id " + Id);
            }
            var imageUrls = await imageRepository.GetAllURLs(rentalObject.EnglishTitle);
            RentalObjectDTO rentalObjectDTO = rentalObjectMapper.Map(rentalObject, imageUrls);
            Console.WriteLine(rentalObjectDTO);
            return rentalObjectDTO;
        }


        /// <summary>
        /// Adds new rentalobject to table storage -> creates blob container -> uploads pictures
        /// </summary>
        /// <param name="rentalObjectDRO"></param>
        /// <returns>Statuscode</returns>
        [HttpPost]
        public async Task<IActionResult> Add([FromForm]RentalObjectDRO rentalObjectDRO) 
        {   
            // Check if there are images
            if (rentalObjectDRO.Pictures != null && rentalObjectDRO.Pictures.Any())
            {
                foreach (var img in rentalObjectDRO.Pictures)
                {
                    await imageRepository.UploadAsync(img.OpenReadStream(), Path.GetFileName(img.FileName), rentalObjectDRO.EnglishTitle.ToLower());
                }
            }

            // Declare cardPictureUrl blank in case of no cardPicture
            string cardPictureUrl = "";
            if (rentalObjectDRO.CardPicture != null)
            {
                await imageRepository.UploadAsync(rentalObjectDRO.CardPicture.OpenReadStream(), Path.GetFileName(rentalObjectDRO.CardPicture.FileName), rentalObjectDRO.EnglishTitle.ToLower());
                cardPictureUrl = await imageRepository.GetURL(rentalObjectDRO.EnglishTitle.ToLower(), Path.GetFileName(rentalObjectDRO.CardPicture.FileName));
            } 
            
            // Loop AllocationPeriods and add to storage
            if (rentalObjectDRO.AllocationPeriods != null && rentalObjectDRO.AllocationPeriods.Any()){
                foreach (var period in rentalObjectDRO.AllocationPeriods)
                {
                    var allocationPeriod = allocationPeriodMapper.Map(period, rentalObjectDRO.EnglishTitle);
                    await allocationPeriodRepository.Add(allocationPeriod);

                    var nextYearPeriod = allocationPeriodRepository.AddYear(allocationPeriod, 1);
                    nextYearPeriod.Id = Guid.NewGuid().ToString();
                    await allocationPeriodRepository.Add(nextYearPeriod);
                }
            }

            // Create RentalObject from RentalObjectDRO and upload
            RentalObject rentalObject = rentalObjectMapper.Map(rentalObjectDRO, cardPictureUrl);
            await rentalObjectRepository.Add(rentalObject);

            // Return OK
            return StatusCode(201);
        }

        /// <summary>
        /// Updates rentalObject in database
        /// </summary>
        /// <param name="rentalObjectDRO"></param>
        /// <returns></returns>
        [HttpPut]
        public async Task<IActionResult> Update([FromForm]RentalObjectDRO rentalObjectDRO) 
        {   
            // Remove allocationPeriods since they will be uploaded again
            var existingAllocationPeriods = await allocationPeriodRepository.GetByRentalObject(rentalObjectDRO.EnglishTitle);
            if (existingAllocationPeriods != null && existingAllocationPeriods.Any())
            {
                foreach (var period in existingAllocationPeriods)
                {
                    await allocationPeriodRepository.Delete(period.Id);
                }
            }

            // Get container URI for removal from image URLs
            var containerUri = await imageRepository.GetContainerURL(rentalObjectDRO.EnglishTitle);

            // If no new card picture and no existing card picture, remove from blob and table column
            if (string.IsNullOrEmpty(rentalObjectDRO.ExistingCardPicture)) 
            {
                // Get old image url by getting old table entitiy
                var oldObject = await rentalObjectRepository.GetById(rentalObjectDRO.EnglishTitle);
                if (!string.IsNullOrEmpty(oldObject.CardPicture)) {
                    // Remove container URI from image URL to get fileName
                    //   remove starting at index 1, length of container URI + 1 to include /
                    var fileName = oldObject.CardPicture.Remove(0, containerUri.Length + 1);

                    await imageRepository.DeleteFile(rentalObjectDRO.EnglishTitle, fileName);
                }
            }

            // Do standard rentalObject add
            await Add(rentalObjectDRO);
        
            // Loop pictures not deleted (existingPictures) to compare to stored, delete where not overlap
            var imageUrls = await imageRepository.GetAllURLs(rentalObjectDRO.EnglishTitle);
            var imageUrlList = imageUrls.ToList();
            var newCardPictureUrl = containerUri + '/' + rentalObjectDRO.CardPicture.FileName;
            foreach (var url in imageUrlList)
            {
                if (!rentalObjectDRO.ExistingPictures.Contains(url) 
                    && url != rentalObjectDRO.ExistingCardPicture 
                    && url != newCardPictureUrl)
                {
                    // Remove container URI from image URL to get fileName
                    //   remove starting at index 1, length of container URI + 1 to include /
                    var fileName = url.Remove(0, containerUri.Length + 1);
                    await imageRepository.DeleteFile(rentalObjectDRO.EnglishTitle, fileName);
                }
            }
            
            // Return OK
            return StatusCode(201);
        }

        /// <summary>
        /// Deletes the rentalObject with id
        /// </summary>
        /// <param name="id">englishtitle of the object</param>
        /// <returns>statuscode</returns>
        [HttpDelete("{id}")]
        public async Task<IActionResult> Delete([FromRoute]string id) 
        {
            // Delete allocationPeriods
            var allocationPeriodIds = await allocationPeriodRepository.GetByRentalObject(id);
            if (allocationPeriodIds != null && allocationPeriodIds.Any())
            {
                foreach (var period in allocationPeriodIds) 
                {
                    await allocationPeriodRepository.Delete(period.Id);
                }
            }

            // Delete blobs
            var urls = await imageRepository.GetAllURLs(id);
            if (urls != null && urls.Any())
            {
                await imageRepository.Delete(id);
            }

            await rentalObjectRepository.Delete(id);

            // Return OK
            return StatusCode(200, id + " was deleted");
        }
    }
} 