using communicate_rental_backend_dev.Models.DataTransfer;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.Database;
using System.Linq;

namespace communicate_rental_backend_dev.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class UserController : Controller
    {
        private readonly IUserRepository userRepository;
        private readonly IUserMapper userMapper;
        private readonly IGraphService graphService;
        
        // What happens here?
        //  Send calls to itemRepository
        //  which calls clients that access the data
        //  and returns it here

        public UserController(
            IUserRepository userRepository,
            IUserMapper userMapper,
            IGraphService graphService)
        {
            this.userRepository = userRepository;
            this.userMapper = userMapper;
            this.graphService = graphService;
        }

        /// <summary>
        /// Adds user
        /// </summary>
        /// <param name="userDRO"></param>
        /// <returns>Statuscode of operation</returns>
        [HttpPost]
        public async Task<IActionResult> AddUser([FromBody]UserDRO userDRO) 
        {
            string id = await graphService.GetUserId(userDRO.Email);
            if (id == null) {
                return NotFound("Cannot find Microsoft user with that email.");
            }

            var user = userMapper.Map(userDRO, id);

            await userRepository.Upload(user);
            return Ok();
        }

        /// <summary>
        /// Get single user from DB/Graph
        /// </summary>
        /// <param name="Id">User id</param>
        /// <returns>User DTO for single user</returns>
        [HttpGet]
        [Route("{id}")]
        public async Task<ActionResult<UserDTO>> GetUser([FromRoute]string Id)
        {
            var user = await userRepository.GetById(Id);
            if (user == null)
            {
                return null;
            }

            return await GetCompletedUser(user);
        }

        /// <summary>
        /// Get all users in db
        /// </summary>
        /// <returns>List of users</returns>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<UserDTO>>> GetUsers() 
        {
            var users = await userRepository.GetAll();
            if (users == null || !users.Any())
            {
                return NotFound("Could not find any users.");
            } 

            var tasks = users.Select(user => GetCompletedUser(user));
            return await Task.WhenAll(tasks);
        }

        [HttpDelete]
        [Route("{email}")]
        public async Task<ActionResult<bool>> Delete([FromRoute]string email)
        {
            var userId = await graphService.GetUserId(email);

            if (await userRepository.Delete(userId))
            {
                return Ok(true); 
            }
            return StatusCode(500, false);
        }

        private async Task<UserDTO> GetCompletedUser(User user)
        {
            var graphUser = await graphService.GetGraphUser(user.UserId);
            return userMapper.Map(user, graphUser);
        }
    }
} 