using System;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Models.Common;
using System.Collections.Generic;
using System.Linq;

namespace communicate_rental_backend_dev.Mappers 
{

    public class RentalObjectMapper : IRentalObjectMapper 
    {
        private readonly IAllocationPeriodRepository allocationPeriodRepository;
        private readonly IMapperUtility mapperUtility;

        public RentalObjectMapper(
            IAllocationPeriodRepository allocationPeriodRepository,
            IMapperUtility mapperUtility)
        {
            this.allocationPeriodRepository = allocationPeriodRepository;
            this.mapperUtility = mapperUtility;
        }

        /// <summary>
        /// Creates a rentalObjectDTO
        /// Maps the rentalObject Db model to DTO by transferring imageUrl list to array
        /// and allocationPeriodDTO objects from tablestorage
        /// </summary>
        /// <param name="rentalObject">database model</param>
        /// <param name="imageURLs">list of image urls</param>
        /// <returns>DTO rentalObject</returns>
        public RentalObjectDTO Map(RentalObject rentalObject, IEnumerable<string> imageURLs) 
        {
            if (rentalObject == null) 
            {
                return null;
            }

            return new RentalObjectDTO 
            {
                Address = rentalObject.Address,
                AllocationPeriods = mapperUtility.DeSerializeToArray<AllocationPeriodDTO>(rentalObject.AllocationPeriods),
                ApplicationLimit = rentalObject.ApplicationLimit,
                Bedrooms = rentalObject.Bedrooms, 
                Beds = rentalObject.Beds, 
                CardPicture = rentalObject.CardPicture, 
                CheckList = mapperUtility.DeSerializeToArray<string>(rentalObject.CheckList),
                Description = rentalObject.Description, 
                Directions = rentalObject.Directions,
                DisabledDays =  mapperUtility.DeSerializeToArray<int>(rentalObject.DisabledDays),
                EnglishTitle = rentalObject.EnglishTitle, 
                FireAlarm = mapperUtility.DeSerializeToArray<string>(rentalObject.FireAlarm),
                FireAlarmNoFireInstructions = rentalObject.FireAlarmNoFireInstructions, 
                FireDepartmentNumber = rentalObject.FireDepartmentNumber, 
                FireEquipment = rentalObject.FireEquipment,
                FireResponsible = rentalObject.FireResponsible,
                FireTestResponsible = rentalObject.FireTestResponsible, 
                Equipment = rentalObject.Equipment,
                Latitude = rentalObject.Latitude,
                Longtitude = rentalObject.Longtitude,
                OtherRules = mapperUtility.DeSerializeToArray<string>(rentalObject.OtherRules), 
                OtherPractialInformation = rentalObject.OtherPractialInformation,
                PeriodPrices = mapperUtility.DeSerializeToArray<PeriodPrice>(rentalObject.PeriodPrices), 
                PetsAllowed = rentalObject.PetsAllowed, 
                // Adds array of image urls if exists, otherwise creates empty list
                Pictures = imageURLs != null ? imageURLs.ToArray() : new string[0],
                RentalPeriodsInnDay = rentalObject.RentalPeriodsInnDay,
                RentalPeriodsInnTime = rentalObject.RentalPeriodsInnTime != null ? TimeSpan.Parse(rentalObject.RentalPeriodsInnTime) : TimeSpan.Zero,
                RentalPeriodsOutDay = rentalObject.RentalPeriodsOutDay,
                RentalPeriodsOutTime = rentalObject.RentalPeriodsOutTime != null ? TimeSpan.Parse(rentalObject.RentalPeriodsOutTime) : TimeSpan.Zero,
                RentalPeriods = mapperUtility.DeSerializeToArray<string>(rentalObject.RentalPeriods), 
                BeforeYouTravel = mapperUtility.DeSerializeToArray<BeforeYouTravel>(rentalObject.BeforeYouTravel), 
                Sleeps = rentalObject.Sleeps, 
                SmokingAllowed = rentalObject.SmokingAllowed, 
                Title = rentalObject.Title, 
            };
        }
            
           /// <summary>
           /// serialize array properties and adds url to cardpicture
           /// </summary>
           /// <param name="rentalObjectDRO"></param>
           /// <param name="url"></param>
           /// <returns></returns>
            public RentalObject Map(RentalObjectDRO rentalObjectDRO, string url)
        {
            if (rentalObjectDRO == null)
            {
                return null;
            }

            // Set encoding type to allow all unicode characters to not break Norwegian
            var options = mapperUtility.GetEncoderSettings();

            var otherRules = rentalObjectDRO.OtherRules.Where(x => !string.IsNullOrEmpty(x));
            var checklist = rentalObjectDRO.CheckList.Where(x => !string.IsNullOrEmpty(x));
            var FireAlarm = rentalObjectDRO.FireAlarm.Where(x => !string.IsNullOrEmpty(x));
            var rentalPeriodsOther = rentalObjectDRO.RentalPeriods.Where(x => !string.IsNullOrEmpty(x));
            var periodPrices = rentalObjectDRO.PeriodPrices.Where(x => !string.IsNullOrEmpty(x.Period) && !string.IsNullOrEmpty(x.Price));
            var beforeYouTravel = rentalObjectDRO.BeforeYouTravel.Where(x => !string.IsNullOrEmpty(x.Title) && !string.IsNullOrEmpty(x.Description));
            // var rentalPeriods = rentalObjectDRO.RentalPeriods.Where(x => )

            return new RentalObject
            { 
                Address = rentalObjectDRO.Address,
                AllocationPeriods = mapperUtility.ToJson(rentalObjectDRO.AllocationPeriods),
                ApplicationLimit = rentalObjectDRO.ApplicationLimit,
                Bedrooms = rentalObjectDRO.Bedrooms,
                Beds = rentalObjectDRO.Beds,
                CardPicture = url,
                CheckList = mapperUtility.ToJson(checklist),
                Description = rentalObjectDRO.Description,
                Directions = rentalObjectDRO.Directions,
                DisabledDays = mapperUtility.ToJson(rentalObjectDRO.DisabledDays),
                EnglishTitle = rentalObjectDRO.EnglishTitle,
                FireAlarm = mapperUtility.ToJson(FireAlarm),
                FireAlarmNoFireInstructions = rentalObjectDRO.FireAlarmNoFireInstructions,
                FireDepartmentNumber = rentalObjectDRO.FireDepartmentNumber,
                FireEquipment = rentalObjectDRO.FireEquipment,
                FireResponsible = rentalObjectDRO.FireResponsible,
                FireTestResponsible = rentalObjectDRO.FireTestResponsible,
                Equipment = rentalObjectDRO.Equipment,
                Latitude = rentalObjectDRO.Latitude,
                Longtitude = rentalObjectDRO.Longtitude,
                OtherPractialInformation = rentalObjectDRO.OtherPractialInformation,
                OtherRules = mapperUtility.ToJson(otherRules),
                PeriodPrices = mapperUtility.ToJson(periodPrices),
                PetsAllowed = rentalObjectDRO.PetsAllowed,
                RentalPeriodsInnDay = rentalObjectDRO.RentalPeriodsInnDay,
                RentalPeriodsInnTime = rentalObjectDRO.RentalPeriodsInnTime.ToString(),
                RentalPeriodsOutDay = rentalObjectDRO.RentalPeriodsOutDay,
                RentalPeriodsOutTime = rentalObjectDRO.RentalPeriodsOutTime.ToString(),
                RentalPeriods = mapperUtility.ToJson(rentalPeriodsOther),
                BeforeYouTravel = mapperUtility.ToJson(beforeYouTravel),
                Sleeps = rentalObjectDRO.Sleeps,
                SmokingAllowed = rentalObjectDRO.SmokingAllowed,
                Title = rentalObjectDRO.Title,
            };
        }
    }
}