using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Models.Database;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace communicate_rental_backend_dev
{
    public class BookingRepository : IBookingRepository
    {
        private readonly ITableStorageClient<Booking> client;
        private readonly IHolidayRepository holidayRepository;

        public BookingRepository(ITableStorageClient<Booking> client,
        IHolidayRepository holidayRepository)
        {
            this.client = client;
            this.holidayRepository = holidayRepository;
        }

        public async Task<IEnumerable<Booking>> GetAll()
        {
            return await client.GetAll();
        }

        public async Task<Booking> GetById(string englishTitle, string id)
        {
            return await client.GetById(englishTitle, id);
        }

        public async Task<IEnumerable<Booking>> GetByUserId(string userId)
        {
            return await client.GetByPredicate(x => x.UserId == userId);
        } 

        // Returns for rental objects that are confirmed and after today
        public async Task<IEnumerable<Booking>> GetByRentalObject(string EnglishTitle)
        {
            return await client.GetByPredicate(
                x => x.EnglishTitle == EnglishTitle && 
                x.IsConfirmed == true && 
                x.EndDate >= DateTime.Today
                ); 
        }

        public async Task<IEnumerable<Booking>> GetByRentalObjectInAllocationPeriod(AllocationPeriod allocationPeriod)
        {
            return await client.GetByPredicate(
                x => x.EnglishTitle == allocationPeriod.RentalObject && 
                x.StartDate >= allocationPeriod.AllocationStart && 
                x.StartDate < allocationPeriod.AllocationEnd
                ); 
        }

        public async Task<bool> Add(Booking booking)
        {
            try
            {
                await client.Upsert(booking, booking.Id, booking.EnglishTitle);
                return true;
            }
            catch (System.Exception)
            {
                return false; 
            }
        }

        public async Task<bool> Delete(string englishTitle, string bookingId)
        {
            try
            {
                await client.Delete(englishTitle, bookingId);
                return true;
            }
            catch (System.Exception)
            {
                return false; 
            }

        }

        public async Task<IEnumerable<Booking>> GetAllocatedBookingsOnLocation(string englishTitle)
        {
            return await client.GetByPredicate
            (x => x.IsConfirmed == true && x.EnglishTitle == englishTitle);
        }

        public async Task<IEnumerable<Booking>> GetAllocatedBookingsForUser(string userId)
        {
            return await client.GetByPredicate
            (x => x.IsConfirmed == true && x.UserId == userId);
        }

        // Check if booking is in holiday
        public async Task<bool> IsBookingInHoliday(DateTime startDate, DateTime endDate)
        {
            bool isInHoliday = false;

            // Get holidays in db for rentalPeriod
            var holidays = await holidayRepository.GetByPredicate(startDate, endDate);       
            if (holidays != null && holidays.Any())
            {
                isInHoliday = true;
            }                   
            return isInHoliday;
        }

        /// <summary>
        /// Get all bookings that is confirmed
        /// </summary>
        /// <returns></returns>
        public async Task<IEnumerable<Booking>> GetConfirmedBookings()
        {
            return await client.GetByPredicate
            (x => x.IsConfirmed == true); 
        }

        public async Task<IEnumerable<Booking>> GetBookingsPastEndDate(DateTime today)
        {
            return await client.GetByPredicate(
                x => x.EndDate.Date <= today.Date
                ); 
        }
    }
}