using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Mappers;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Nager.Date;
using System.Globalization;
using communicate_rental_backend_dev.Models.Database;
using System.Linq;

namespace communicate_rental_backend_dev
{

    public class HolidayRepository : IHolidayRepository
    {
        private readonly ITableStorageClient<Holiday> client;
        private readonly IHolidayMapper holidayMapper;

        public HolidayRepository(ITableStorageClient<Holiday> client,
        IHolidayMapper holidayMapper)
        {
            this.client = client;
            this.holidayMapper = holidayMapper;
        }

        // Get holiday by Id
        public async Task<Holiday> GetHoliday(string id)
        {
            return await client.GetById(id, id);
        }

        public async Task<IEnumerable<Holiday>> GetAll()
        {
            return await client.GetAll();
        }

        // Adds holidays passed from frontend
        public async Task Add(Holiday holiday)
        {
            await client.Upsert(holiday, holiday.Id, holiday.Id);
        }

        public async Task Delete(string id)
        {
            await client.Delete(id, id);
        }

        /// <summary>
        /// Get holidays in database for certain year
        /// </summary>
        /// <param name="year">int</param>
        /// <returns>list of holidays in "year"</returns>
        public async Task<IEnumerable<Holiday>> GetByPredicate(int year)
        {
            return await client.GetByPredicate(x=>
            x.DateOfHoliday.Year == year);
        }

        /// <summary>
        /// Get all holidays within a year
        /// </summary>
        /// <param name="startDate"></param>
        /// <param name="endDate"></param>
        /// <returns></returns>
        public async Task<IEnumerable<Holiday>> GetByPredicate(DateTime startDate, DateTime endDate)
        {
            return await client.GetByPredicate(x=>
            x.DateOfHoliday.Date >= startDate.Date &&
            x.DateOfHoliday.Date <= endDate.Date);
        }

        /// <summary>
        /// Adds public holidays for specified year if they dont exist in db
        /// includes nager.date and week 8/40 for year specified 
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddPublicHolidaysInYearIfNotExisists(int year)
        {
            // if all holidays could be added
            bool status = true;

            var holidays = GetPublicHolidays(year);
            var existingHolidays = await GetAll();

            foreach (var day in holidays)
            {
                day.DateOfHoliday.ToUniversalTime(); 
                bool matchingDay = false;

                if (existingHolidays != null && existingHolidays.Any()) {
                    foreach (var d in existingHolidays)
                    {
                        if (day.DateOfHoliday == d.DateOfHoliday) { matchingDay = true; }
                        if (matchingDay != true) 
                        {
                            await Add(day);
                            Console.WriteLine(day + "added to holiday database."); 
                        }
                        else { status = false; }
                    }  
                }
            }
            return status;
        }

        /// <summary>
        /// Put together holidays from week function and public holidays
        /// </summary>
        /// <param name="year"></param>
        /// <returns></returns>
        public IEnumerable<Holiday> GetPublicHolidays(int year)
        {

            var publicHolidaysInYear = DateSystem.GetPublicHoliday(year, CountryCode.NO);
            var holidaysInWeek = new List<Holiday>();
            holidaysInWeek.AddRange(AddWeekAsHoliday(40, year));
            holidaysInWeek.AddRange(AddWeekAsHoliday(8, year));
            
            var holidays = new List<Holiday>();

            foreach (var publicHoliday in publicHolidaysInYear)
            {
                var holiday = holidayMapper.Map(publicHoliday.Date);
                holidays.Add(holiday);
            }

            foreach (var weekHoliday in holidaysInWeek)
            {
                foreach (var day in holidays)
                {
                    if (day.DateOfHoliday.Date != weekHoliday.DateOfHoliday.Date)
                    {
                        holidays.Add(weekHoliday);
                    }
                }
            }

            return holidays;
        }

        /// <summary>
        /// Finds first monday in year, and calculated monday in week
        /// iterates over the next six days to add all days in week
        /// </summary>
        /// <param name="weekNumber">to be calculated</param>
        /// <param name="year"></param>
        /// <returns></returns>
        public IEnumerable<Holiday> AddWeekAsHoliday(int weekNumber, int year)
        {
            DateTime mondayInWeek1 = ISOWeek.GetYearStart(year);
            var mondayInWeek = mondayInWeek1.AddDays(7*weekNumber);

            var holidaysInWeek = new List<Holiday>();

            for (var day=mondayInWeek; day<=mondayInWeek.AddDays(6); day = day.AddDays(1)) 
            {
                var holiday = new Holiday {
                Id = Guid.NewGuid().ToString(),
                DateOfHoliday = day.Date.ToUniversalTime()
                };
                holidaysInWeek.Add(holiday);
            }
            return holidaysInWeek;
        }
    }
}