using communicate_rental_backend_dev.Clients;
using communicate_rental_backend_dev.Configurations;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using communicate_rental_backend_dev.BusinessLogic.Services;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Server.IISIntegration;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;
using System.Reflection;
using System;
using System.IO;

namespace communicate_rental_backend_dev
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers();
            services.AddSwaggerGen(c =>
            {
                c.SwaggerDoc("v1", new OpenApiInfo { Title = "communicate_rental_backend_dev", Version = "v1" });
                
                // Set the comments path for the Swagger JSON and UI.
                var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
                var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
                c.IncludeXmlComments(xmlPath);
            });

            services.AddCors(options =>
            {
                options.AddPolicy("CorsPolicy",
                builder => builder.WithOrigins(Configuration.GetValue<string>("CORSpolicy"))
                .AllowAnyMethod()
                .AllowAnyHeader()
                .AllowCredentials());
            });
            services.AddAuthentication(IISDefaults.AuthenticationScheme);
            
            var storageAccountConnectionString = Configuration.GetConnectionString("DefaultConnection");
            
            // BlobStorage clients
            // As long as we always uploads and read data as file streams, this is enough for all data to be stored as blobs
            services.AddSingleton<IBlobStorageClient>(BlobStorageClientFactory.Create(storageAccountConnectionString));

            // Repositories for BlobStorage (One for each separate 'type' of blob)
            services.AddSingleton<IImageRepository, ImageRepository>();
            services.AddSingleton<IBlobFeedbackRepository, BlobFeedbackRepository>();
            services.AddSingleton<IBlobBookingRepository, BlobBookingRepository>();

            // DbModel clients for TableStorage 
            services.AddSingleton(TableStorageClientFactory.Create<AllocationPeriod>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<Booking>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<Checklist>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<Holiday>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<RentalObject>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<User>(storageAccountConnectionString));
            services.AddSingleton(TableStorageClientFactory.Create<Feedback>(storageAccountConnectionString));

            // Repositories for TableStorage
            services.AddSingleton<IAllocationPeriodRepository, AllocationPeriodRepository>();
            services.AddSingleton<IBookingRepository, BookingRepository>();
            services.AddSingleton<IChecklistRepository, ChecklistRepository>();
            services.AddSingleton<IHolidayRepository, HolidayRepository>();
            services.AddSingleton<IRentalObjectRepository, RentalObjectRepository>();
            services.AddSingleton<IUserRepository, UserRepository>();
            services.AddSingleton<IFeedbackRepository, FeedbackRepository>();

            // Configurations
            services.AddSingleton<IAzureAdConfig>(Configuration.GetSection("AzureAdConfig").Get<AzureAdConfig>());
            services.AddSingleton<IEmailConfig>(Configuration.GetSection("EmailConfig").Get<EmailConfig>());

            // Services
            services.AddSingleton<IEmailService, EmailService>();
            services.AddSingleton<IGraphService, GraphService>();
            services.AddSingleton<IAllocationService, AllocationService>();
            services.AddSingleton<IFunctionAppService, FunctionAppService>();
 
            // HttpClient
            services.AddHttpClient<IGraphService, GraphService>();

            // Mappers
            services.AddSingleton<IAllocationPeriodMapper, AllocationPeriodMapper>(); 
            services.AddSingleton<IBookingMapper, BookingMapper>();
            services.AddSingleton<IChecklistMapper, ChecklistMapper>();
            services.AddSingleton<IHolidayMapper, HolidayMapper>();
            services.AddSingleton<IMapperUtility, MapperUtility>();
            services.AddSingleton<IRentalObjectMapper, RentalObjectMapper>();
            services.AddSingleton<IUserMapper, UserMapper>();
            services.AddSingleton<IFeedbackMapper, FeedbackMapper>();
            
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                app.UseSwagger();
                app.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json", "communicate_rental_backend_dev v1"));
            }

            app.UseCors("CorsPolicy");
            
            app.UseHttpsRedirection();

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}