using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using FluentAssertions;
using MimeKit;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace unit_tests
{
    [TestClass]
    public class AllocationServiceTests
    {
        private Mock<IBookingRepository> bookingRepositoryMock;
        private Mock<IBookingMapper> bookingMapperMock;
        private Mock<IEmailService> emailServiceMock;
        private Mock<IGraphService> graphServiceMock;
        private Mock<IUserRepository> userRepositoryMock;
        private Mock<IBlobBookingRepository> blobBookingMock;
        private IAllocationService service;
        private Fixture fixture;


        [TestInitialize]
        public void TestInit()
        {
            // Set up mock dependencies
            bookingRepositoryMock = new Mock<IBookingRepository>();
            bookingMapperMock = new Mock<IBookingMapper>();
            emailServiceMock = new Mock<IEmailService>();
            graphServiceMock = new Mock<IGraphService>();
            userRepositoryMock = new Mock<IUserRepository>();
            blobBookingMock = new Mock<IBlobBookingRepository>();
            service = new AllocationService
            (
                // Add .Object to make them correct type for class/constructor
                bookingRepositoryMock.Object,
                bookingMapperMock.Object,
                emailServiceMock.Object,
                graphServiceMock.Object,
                userRepositoryMock.Object,
                blobBookingMock.Object
            );

            // Set up fixture for object creation
            fixture = new Fixture();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public async Task Allocate_NullBookingsInAllocationPeriod_ArgumentNullException()
        {
            // Arrange
            var input = fixture.Create<AllocationPeriod>();
            var bookings = fixture.Create<IEnumerable<Booking>>();
            var user = fixture.Create<User>();
            var email = fixture.Create<MailboxAddress>();

            bookingRepositoryMock
            .Setup(mock => mock.GetByRentalObjectInAllocationPeriod(It.IsAny<AllocationPeriod>())).ReturnsAsync((IEnumerable<Booking>)null);
            userRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(user);
            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(email);
            emailServiceMock
            .Setup(mock => mock.SendBookingConfirmation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);
            
            // Act/Assert
            await service.Allocate(input);
        }

        [TestMethod]
        [DataRow(1, false, 0, true)]
        [DataRow(0, false, 0, true)]
        [DataRow(2, true, 0, true)]
        [DataRow(1, true, 0, true)]
        [DataRow(3, false, 3, false)]
        public async Task Allocate_Userpoints_ShouldEqualExpectedReturn(int points, bool holiday, int expectedPoints, bool expectedResult)
        {
            // Arrange
            var input = fixture.Create<AllocationPeriod>();
            var bookings = fixture.Create<IEnumerable<Booking>>();
            var user = fixture.Create<User>();
            var email = fixture.Create<MailboxAddress>();

            bookingRepositoryMock
            .Setup(mock => mock.GetByRentalObjectInAllocationPeriod(It.IsAny<AllocationPeriod>())).ReturnsAsync(bookings);
            userRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(user);
            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(email);
            emailServiceMock
            .Setup(mock => mock.SendBookingConfirmation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);
            
            // Set up test data
            var allocatedBooking = bookings.First();
            allocatedBooking.UserId = user.UserId;
            user.Points = points;
            allocatedBooking.IsInHoliday = holiday;
            allocatedBooking.IsConfirmed = false;
            
            // Act
            await service.Allocate(input);

            // Assert
            expectedResult.Should().Be(expectedPoints.Equals(user.Points));
        }

        
        [TestMethod]
        [DataRow(0, 1, 2, 2)]
        [DataRow(1, 2, 0, 1)]
        [DataRow(2, 0, 1, 0)]
        public async Task GetBookingWithHighestPriority_ListOfBooking_ShouldReturnExpected(int user1Points, int user2Points, int user3Points, int expectedPlace)
        {
            // Arrange
            var input = fixture.Create<List<Booking>>();
            var users = fixture.Create<List<User>>();

            var user1 = users.ElementAt(0);
            var user2 = users.ElementAt(1);
            var user3 = users.ElementAt(2);

            user1.Points = user1Points;
            user2.Points = user2Points;
            user3.Points = user3Points;


            userRepositoryMock
            .Setup(mock => mock.GetById(input.First().UserId)).ReturnsAsync(user1);
            userRepositoryMock
            .Setup(mock => mock.GetById(input.ElementAt(1).UserId)).ReturnsAsync(user2);
            userRepositoryMock
            .Setup(mock => mock.GetById(input.ElementAt(2).UserId)).ReturnsAsync(user3);
            
            // Act
            var result = await service.GetBookingWithHighestPriority(input);

            // Assert
            result.Should().Be(input.ElementAt(expectedPlace));   
        }

        [TestMethod]
        [DataRow(2, 2, 3, 0)]
        [DataRow(2, 0, 3, 1)]
        [DataRow(2, -1, 0, 0)]
        [DataRow(3, 1, 3, 1)]
        public async Task DeleteBookingsWithConflicts_Bookings_ShouldBeExpectedResult(int daysFromAllocatedStart, int comparedStart, int comparedEnd, int expectedTimesExecuted)
        {
            // Arrange
            var allocatedBooking = fixture.Create<Booking>();
            var comparedObject = fixture.Create<Booking>();
            var email = fixture.Create<MailboxAddress>();

            allocatedBooking.StartDate = DateTime.Today.ToUniversalTime();
            allocatedBooking.EndDate = DateTime.Today.AddDays(daysFromAllocatedStart).ToUniversalTime();

            comparedObject.StartDate = DateTime.Today.AddDays(comparedStart).ToUniversalTime();
            comparedObject.EndDate = DateTime.Today.AddDays(comparedEnd).ToUniversalTime();

            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(email);
            emailServiceMock
            .Setup(mock => mock.SendBookingCancellation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);
            blobBookingMock
            .Setup(mock => mock.UploadAsyncDeletedBooking(It.IsAny<Booking>())).ReturnsAsync(true);
            bookingRepositoryMock
            .Setup(mock => mock.Delete(It.IsAny<string>(), It.IsAny<string>())).ReturnsAsync(true);
            

            // Act
            await service.DeleteBookingsWithConflicts(allocatedBooking, comparedObject);

            // Assert
            bookingRepositoryMock
            .Verify(mock => mock.Delete(It.IsAny<string>(), It.IsAny<string>()), Times.Exactly(expectedTimesExecuted));
        }
    }
}