using System.Xml.Xsl;
using System.Threading.Tasks.Dataflow;
using System.Reflection.Metadata;
using System;
using System.Xml.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Reflection.Metadata.Ecma335;
using System.Runtime.CompilerServices;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using unit_tests.Utilities;
using MimeKit;

namespace unit_tests
{
    [TestClass]
    public class BookingControllerTests
    {
        private Mock<IAllocationPeriodRepository> allocationPeriodRepositoryMock;
        private Mock<IBookingRepository> bookingRepositoryMock;
        private Mock<IBookingMapper> bookingMapperMock;
        private Mock<IGraphService> graphServiceMock;
        private Mock<IEmailService> emailServiceMock;
        private BookingsController controller;
        private Mock<IAllocationService> allocationServiceMock;
        private Mock<IUserRepository> userRepositoryMock;
        private Fixture fixture;

        [TestInitialize]
        public void TestInit()
        {
            // Set up Mock dependencies
            allocationPeriodRepositoryMock = new Mock<IAllocationPeriodRepository>();
            bookingRepositoryMock = new Mock<IBookingRepository>();
            bookingMapperMock = new Mock<IBookingMapper>();
            graphServiceMock = new Mock<IGraphService>();
            emailServiceMock = new Mock<IEmailService>();
            allocationServiceMock = new Mock<IAllocationService>();
            userRepositoryMock = new Mock<IUserRepository>();

            controller = new BookingsController
            (
                allocationPeriodRepositoryMock.Object,
                bookingRepositoryMock.Object,
                bookingMapperMock.Object,
                emailServiceMock.Object,
                graphServiceMock.Object,
                allocationServiceMock.Object,
                userRepositoryMock.Object
            );

            fixture = new Fixture();
        }

        [TestMethod]
        [DataRow(1, 3, 202)]
        [DataRow(2, 1, 202)]
        [DataRow(3, -1, 201)]
        [DataRow(2, 10, 202)]
        [DataRow(2, 0, 201)]
        public async Task Add_ShouldAssign_ShouldReturnExpected(int endDay, int periodStart, int expected)
        {
            // Arrange
            var input = fixture.Create<BookingDRO>();
            input.StartDate = DateTime.Today.ToUniversalTime();
            input.EndDate = input.StartDate.AddDays(endDay);

            // Setup booking data
            var mappedBooking = fixture.Create<Booking>();
            bookingMapperMock
            .Setup(mock => mock.Map(It.IsAny<BookingDRO>())).Returns(mappedBooking);
            mappedBooking.StartDate = DateTime.Today.ToUniversalTime();
            mappedBooking.EndDate = mappedBooking.StartDate.AddDays(endDay);
            mappedBooking.IsConfirmed = false;
            mappedBooking.IsInHoliday = false;

            // Setup mock data
            var currentPeriodMock = fixture.Create<List<AllocationPeriod>>();
            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetCurrent(It.IsAny<DateTime>(), It.IsAny<string>())).ReturnsAsync(currentPeriodMock);
            var currentPeriod = currentPeriodMock[0];
            currentPeriod.AllocationStart = DateTime.Today.AddDays(periodStart).ToUniversalTime();
            currentPeriod.AllocationEnd = currentPeriod.AllocationStart.AddDays(2);

            // Setup return value for all methods to make sure the right
            // return type is granted
            var email = fixture.Create<MailboxAddress>();
            var uploadResult = fixture.Create<CreatedResult>();
            var user = fixture.Create<User>();

            bookingRepositoryMock
            .Setup(mock => mock.IsBookingInHoliday(It.IsAny<DateTime>(), It.IsAny<DateTime>())).ReturnsAsync(false);
            emailServiceMock
            .Setup(mock => mock.SendBookingConfirmation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);
            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(email);

            userRepositoryMock
            .Setup(mock => mock.Upload(It.IsAny<User>())).ReturnsAsync(uploadResult);
            userRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(user);
            
            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsOnLocation(It.IsAny<string>())).ReturnsAsync((IEnumerable<Booking>)null);
            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsForUser(It.IsAny<string>())).ReturnsAsync((IEnumerable<Booking>)null);
            bookingRepositoryMock
            .Setup(mock => mock.Add(It.IsAny<Booking>())).ReturnsAsync(true);
            

            // Act
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Should().Be(expected);
        }
        

        [TestMethod]
        [DataRow(3, 0, true, 409)]
        [DataRow(3, 3, false, 201)]
        [DataRow(3, 2, true, 409)]
        public async Task Add_BookingInPeriodConflictCheck_ShouldBeExpectedResult(int endOfBooking, int conflictingStart, bool mockReturn, int expectedResult)
        {
            // Arrange
            var input = fixture.Create<BookingDRO>();
            input.StartDate = DateTime.Today.ToUniversalTime();

            var mappedBooking = fixture.Create<Booking>();
            mappedBooking.IsConfirmed = false;
            mappedBooking.IsInHoliday = false;
            mappedBooking.StartDate = DateTime.Today.ToUniversalTime();
            mappedBooking.EndDate = mappedBooking.StartDate.AddDays(endOfBooking);

            var conflictingBookings = fixture.Create<IEnumerable<Booking>>();
            var userBookings = fixture.Create<IEnumerable<Booking>>();

            var currentPeriodMock = fixture.Create<List<AllocationPeriod>>();
            var emailMock = fixture.Create<MailboxAddress>();
            var successResponse = fixture.Create<CreatedResult>();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetCurrent(It.IsAny<DateTime>(), It.IsAny<string>())).ReturnsAsync(currentPeriodMock);
            var currentPeriod = currentPeriodMock[0];
            currentPeriod.AllocationStart = DateTime.Today.ToUniversalTime();
            currentPeriod.AllocationEnd = DateTime.Today.AddDays(14).ToUniversalTime();

            // Setup return value for all methods to make sure the right
            // return type is granted
            bookingMapperMock.SetReturnsDefault(mappedBooking);

            bookingRepositoryMock
            .Setup(mock => mock.IsBookingInHoliday(It.IsAny<DateTime>(), It.IsAny<DateTime>())).ReturnsAsync(false);

            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(emailMock);

            emailServiceMock
            .Setup(mock => mock.SendBookingConfirmation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);

            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsOnLocation(It.IsAny<string>())).ReturnsAsync(conflictingBookings);

            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsForUser(It.IsAny<string>())).ReturnsAsync(userBookings);

            allocationServiceMock
            .Setup(mock => mock.CheckForConflictingBookings(It.IsAny<Booking>(), It.IsAny<Booking>())).Returns(mockReturn);

            var uploadResult = fixture.Create<CreatedResult>();
            var user = fixture.Create<User>();
            userRepositoryMock
            .Setup(mock => mock.Upload(It.IsAny<User>())).ReturnsAsync(uploadResult);
            userRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(user);

            var conflictingBooking = conflictingBookings.ElementAt(0);
            conflictingBooking.StartDate = DateTime.Today.AddDays(conflictingStart).ToUniversalTime();
            conflictingBooking.EndDate = conflictingBooking.StartDate.AddDays(1);

            var otherBooking = conflictingBookings.ElementAt(1);
            otherBooking.StartDate = mappedBooking.EndDate.AddDays(10).ToUniversalTime();
            otherBooking.EndDate = otherBooking.StartDate.AddDays(1);

            var otherBooking1 = conflictingBookings.ElementAt(2);
            otherBooking1.StartDate = mappedBooking.EndDate.AddDays(10).ToUniversalTime();
            otherBooking1.EndDate = otherBooking1.StartDate.AddDays(1);

            // Act
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Should().Be(expectedResult);
        }

        [TestMethod]
        [DataRow(13, true)]
        [DataRow(12, false)]
        [DataRow(11, false)]
        public async Task Add_FutureBooking_ShouldReturnExpected(int month, bool expectedStatusCodeIs409)
        {
            // Arrange
            var input = fixture.Create<BookingDRO>();
            input.StartDate = DateTime.Today.Date.AddMonths(month);

            var mappedBooking = fixture.Create<Booking>();

            var allocationPeriods = fixture.Create<IEnumerable<AllocationPeriod>>();
            var userBookings = fixture.Create<IEnumerable<Booking>>();

            var currentPeriodMock = fixture.Create<List<AllocationPeriod>>();
            var emailMock = fixture.Create<MailboxAddress>();
            var successResponse = fixture.Create<CreatedResult>();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetCurrent(It.IsAny<DateTime>(), It.IsAny<string>())).ReturnsAsync(allocationPeriods);

            bookingMapperMock.SetReturnsDefault(mappedBooking);

            bookingRepositoryMock
            .Setup(mock => mock.IsBookingInHoliday(It.IsAny<DateTime>(), It.IsAny<DateTime>())).ReturnsAsync(false);

            graphServiceMock
            .Setup(mock => mock.GetEmailAsync(It.IsAny<string>())).ReturnsAsync(emailMock);

            emailServiceMock
            .Setup(mock => mock.SendBookingConfirmation(It.IsAny<MailboxAddress>(), It.IsAny<Booking>())).Returns(true);

            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsOnLocation(It.IsAny<string>())).ReturnsAsync((IEnumerable<Booking>)null);
            bookingRepositoryMock
            .Setup(mock => mock.GetAllocatedBookingsForUser(It.IsAny<string>())).ReturnsAsync((IEnumerable<Booking>)null);

            allocationServiceMock
            .Setup(mock => mock.CheckForConflictingBookings(It.IsAny<Booking>(), It.IsAny<Booking>())).Returns(false);

            var uploadResult = fixture.Create<CreatedResult>();
            var user = fixture.Create<User>();
            userRepositoryMock
            .Setup(mock => mock.Upload(It.IsAny<User>())).ReturnsAsync(uploadResult);
            userRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(user);

            // Act
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Equals(409).Should().Be(expectedStatusCodeIs409);
        }

        [TestMethod]
        public void Mapping_InputBooking_ReturnBookingDTO()
        {
            // Arrange
            var input = fixture.Create<Booking>();
            var mappedBooking = fixture.Create<BookingDTO>();

            bookingMapperMock.SetReturnsDefault(mappedBooking);

            // Act
            bookingMapperMock.Object.Map(input);

            // Assert
            bookingMapperMock.Verify(mock => mock.Map(It.IsAny<Booking>()));
        }

        [TestMethod]
        public async Task GetAll_ReturnsListOfBookings()
        {
            // Arrange
            var ListOfBookings = fixture.Create<List<Booking>>();

            bookingRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(ListOfBookings);
            
            // Act
            await bookingRepositoryMock.Object.GetAll();
            
            // Assert
            bookingRepositoryMock.Verify(mock => mock.GetAll());            
        }

        [TestMethod]
        public async Task GetByUserId_InputUserId_RetunsListOfBookings()
        {
            // Arrange
            var input = fixture.Create<string>();

            // Expected result
            var listOfBookings = fixture.Create<List<Booking>>();

            bookingRepositoryMock
            .Setup(mock => mock.GetByUserId(It.IsAny<string>())).ReturnsAsync(listOfBookings);

            // Act
            await bookingRepositoryMock.Object.GetByUserId(input);

            // Assert
             bookingRepositoryMock.Verify(mock => mock.GetByUserId(It.IsAny<string>()));
        }

        [TestMethod]
        public async Task GetByRentalObject_InputEnglishtitle_RetunsListOfBookings()
        {
            // Arrange
            var input = fixture.Create<string>();

            // Expected result
            var listOfBookings = fixture.Create<List<Booking>>();

            bookingRepositoryMock
            .Setup(mock => mock.GetByRentalObject(It.IsAny<string>())).ReturnsAsync(listOfBookings);

            // Act
            await bookingRepositoryMock.Object.GetByRentalObject(input);

            // Assert
            bookingRepositoryMock.Verify(mock => mock.GetByRentalObject(It.IsAny<string>()));
        }

        [TestMethod]
        public async Task Delete_BookingIdAndEnglishtitle_RetunsBoolean()
        {
            // Arrange
            var input = fixture.Create<string>();

            var expectedResult = fixture.Create<bool>();

            bookingRepositoryMock
            .Setup(mock => mock.Delete(It.IsAny<string>(), It.IsAny<string>())).ReturnsAsync(expectedResult);

            // Act
            await bookingRepositoryMock.Object.Delete(input, input);

            // Assert
            bookingRepositoryMock.Verify(mock => mock.Delete(It.IsAny<string>(), It.IsAny<string>()));
        }
    }
}
