using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Threading.Tasks;
using communicate_rental_backend_dev.BusinessLogic.Services;
using communicate_rental_backend_dev.Models.Common;

namespace unit_tests
{
    [TestClass]
    public class ChecklistControllerTests
    {
        private Mock<IChecklistRepository> checklistRepositoryMock;
        private Mock<IChecklistMapper> checklistMapperMock;
        private Mock<IGraphService> graphServiceMock; 
        private Mock<IRentalObjectRepository> rentalObjectRepositoryMock;
        private ChecklistsController controller;
        private Fixture fixture;


        [TestInitialize]
        public void TestInit()
        {
            // Set up mock dependencies
            checklistRepositoryMock = new Mock<IChecklistRepository>();
            checklistMapperMock = new Mock<IChecklistMapper>();
            graphServiceMock = new Mock<IGraphService>();
            rentalObjectRepositoryMock = new Mock<IRentalObjectRepository>();
            controller = new ChecklistsController
            (
                // Add .Object to make them correct type for class/constructor
                checklistRepositoryMock.Object,
                checklistMapperMock.Object, 
                graphServiceMock.Object,
                rentalObjectRepositoryMock.Object
            );

            // Set up fixture for object creation
            fixture = new Fixture();
        }

        [TestMethod]
        public async Task GetChecklists_VerifiesReturnType_ShouldReturnArrayOfDTO()
        {
            // Arrange
            var dbResult = fixture.Create<IEnumerable<Checklist>>();
            var mappedChecklist = fixture.Create<ChecklistDTO>();
            var expectedResult = fixture.Create<ChecklistDTO[]>().GetType();

            checklistRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbResult);
            
            checklistMapperMock
            .Setup(mock => mock.Map(It.IsAny<Checklist>(), It.IsAny<string>())).Returns(mappedChecklist);
            
            // Act
            var result = await controller.GetChecklists();

            // Assert
            result.Value.Should().BeOfType(expectedResult);
        }

        [TestMethod]
        public async Task GetChecklists_VerifiesReturnType_ShouldReturnNull()
        {
            // Arrange
            var expectedResult = fixture.Create<ArgumentNullException>().ParamName;
            var dbReturn = fixture.Create<IEnumerable<Checklist>>();

            checklistRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync((IEnumerable<Checklist>)null);

            // Act
            try 
            {
                var result = await controller.GetChecklists();
            }
            catch (ArgumentException e)
            {
                Assert.AreEqual(expectedResult, e.InnerException);
            }
        }
        
        [TestMethod]
        public async Task Get_VerifiesReturnType_ShouldReturnChecklistDTO()
        {
            // Arrange
            var input = fixture.Create<string>();
            var dbResult = fixture.Create<IEnumerable<Checklist>>();
            var mappedChecklist = fixture.Create<ChecklistDTO>();
            var expectedResult = fixture.Create<ChecklistDTO>().GetType();

            checklistRepositoryMock
            .Setup(mock => mock.GetByBookingId(It.IsAny<string>())).ReturnsAsync(dbResult);
            checklistMapperMock
            .Setup(mock => mock.Map(It.IsAny<Checklist>(), "")).Returns(mappedChecklist);
            
            // Act
            var result = await controller.Get(input, input);

            // Assert
            result.Value.Should().BeOfType(expectedResult);
        }

        [TestMethod]
        public async Task Get_VerifiesReturnType_ShouldReturnEmptyList()
        {
            // Arrange
            var input = fixture.Create<string>();
            var expectedResult = fixture.Create<ChecklistDTO>().GetType();
            var mappedChecklist = fixture.Create<ChecklistDTO>();
            var dbReturn = fixture.Create<Checklist>();
            var array = fixture.Create<string[]>();

            checklistRepositoryMock
            .Setup(mock => mock.GetByBookingId(It.IsAny<string>())).ReturnsAsync((IEnumerable<Checklist>)null);
            rentalObjectRepositoryMock
            .Setup(mock => mock.GetChecklist(It.IsAny<string>())).ReturnsAsync(array);
            checklistMapperMock
            .Setup(mock => mock.Map(It.IsAny<string[]>())).Returns(mappedChecklist);
            

            // Act
            var result = await controller.Get(input, input);
            
            // ASsert
            result.Value.Should().BeOfType(expectedResult);
        }


        [TestMethod]
        public async Task GetChecklists_VerifiesMapper_FromDROToDb()
        {
            // Arrange
            var input = fixture.Create<ChecklistDRO>();
            var mappedChecklist = fixture.Create<Checklist>();

            checklistMapperMock
            .Setup(mock => mock.Map(It.IsAny<ChecklistDRO>())).Returns(mappedChecklist);
            
            // Act
            await controller.UploadChecklist(input);

            // Assert
            checklistMapperMock.Verify(mock => mock.Map(It.IsAny<ChecklistDRO>()));
        }

        [TestMethod]
        public async Task Add_ValidModel_ShouldReturnTrue()
        {
            // Arrange
            var input = fixture.Create<ChecklistDRO>();
            var mappedChecklist = fixture.Create<Checklist>();
            
            checklistMapperMock
            .Setup(mock => mock.Map(It.IsAny<ChecklistDRO>())).Returns(mappedChecklist);

            // Act
            var result = await controller.UploadChecklist(input);
            
            // Assert
            result.Result.Should().BeOfType<CreatedResult>();
        }

        [TestMethod]
        public async Task Delete_IdAsInput_ShouldReturn200()
        {
            // Arrange
            var input = fixture.Create<string>();
            var expectedStatus = 200;

            // Act
            var result = await controller.Delete(input) as ObjectResult;

            // Assert
            result.StatusCode.Value.Should().Be(expectedStatus);
        }
    }
}