using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataReceiving;
using communicate_rental_backend_dev.Models.DataTransfer;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace unit_tests
{
    [TestClass]
    public class HolidayControllerTests
    {
        private Mock<IHolidayRepository> holidayRepositoryMock;
        private Mock<IHolidayMapper> holidayMapperMock;
        private HolidayController controller;
        private Fixture fixture;

        [TestInitialize]
        public void TestInit()
        {
            // Set up Mock dependencies
            holidayRepositoryMock = new Mock<IHolidayRepository>();
            holidayMapperMock = new Mock<IHolidayMapper>();
            
            controller = new HolidayController
            (
                holidayRepositoryMock.Object,
                holidayMapperMock.Object
            );

            fixture = new Fixture();
        }

        [TestMethod]
        public async Task GetHolidays_VerifiesMapping()
        {
            // Arrange
            var dbReturn = fixture.Create<IEnumerable<Holiday>>();
            var mappedHoliday = fixture.Create<HolidayDTO>();

            holidayRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            holidayMapperMock.
            Setup(mock => mock.Map(It.IsAny<Holiday>())).Returns(mappedHoliday);

            // Act
            await controller.GetHolidays();

            // Assert
            holidayMapperMock.Verify(mock => mock.Map(It.IsAny<Holiday>()));
        }

        [TestMethod]
        public async Task Add_HolidayMatchingAndNotMatchingExisting_ShouldReturn201()
        {
            // Arrange
            var input = fixture.Create<HolidayDRO>();

            var mappedHoliday = fixture.Create<List<Holiday>>();
            mappedHoliday.First().DateOfHoliday = DateTime.Today.ToUniversalTime();
            mappedHoliday.Last().DateOfHoliday = DateTime.Today.AddDays(3).ToUniversalTime();
            
            var dbReturn = fixture.Create<IEnumerable<Holiday>>();
            var expectedResult = 201;

            holidayMapperMock
            .Setup(mock => mock.Map(It.IsAny<HolidayDRO>())).Returns(mappedHoliday);

            holidayRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            dbReturn.First().DateOfHoliday = DateTime.Today.ToUniversalTime();
            dbReturn.ElementAt(1).DateOfHoliday = DateTime.Today.AddDays(1).ToUniversalTime();
            dbReturn.ElementAt(2).DateOfHoliday = DateTime.Today.AddDays(2).ToUniversalTime();

            // Act 
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Should().Be(expectedResult);
        }

         [TestMethod]
        public async Task Add_NotMatchingExisting_ShouldReturn201()
        {
            // Arrange
            var input = fixture.Create<HolidayDRO>();

            var mappedHoliday = fixture.Create<List<Holiday>>();
            mappedHoliday.First().DateOfHoliday = DateTime.Today.ToUniversalTime();
            mappedHoliday.Last().DateOfHoliday = DateTime.Today.AddDays(3).ToUniversalTime();

            var expectedResult = 201;

            holidayMapperMock
            .Setup(mock => mock.Map(It.IsAny<HolidayDRO>())).Returns(mappedHoliday);

            holidayRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync((IEnumerable<Holiday>)null);

            // Act 
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Should().Be(expectedResult);
        }

        [TestMethod]
        public async Task Add_MatchingExisting_ShouldReturn500()
        {
            // Arrange
            var input = fixture.Create<HolidayDRO>();
            var expectedResult = 500;

            var mappedHoliday = fixture.Create<List<Holiday>>();
            mappedHoliday.First().DateOfHoliday = DateTime.Today.ToUniversalTime();
            mappedHoliday.ElementAt(1).DateOfHoliday = DateTime.Today.AddDays(1).ToUniversalTime();
            mappedHoliday.Last().DateOfHoliday = DateTime.Today.AddDays(2).ToUniversalTime();
            
            var dbReturn = fixture.Create<IEnumerable<Holiday>>();
            dbReturn.First().DateOfHoliday = DateTime.Today.ToUniversalTime();
            dbReturn.ElementAt(1).DateOfHoliday = DateTime.Today.AddDays(1).ToUniversalTime();
            dbReturn.Last().DateOfHoliday = DateTime.Today.AddDays(2).ToUniversalTime();

            holidayMapperMock
            .Setup(mock => mock.Map(It.IsAny<HolidayDRO>())).Returns(mappedHoliday);

            holidayRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            // Act 
            var result = await controller.Add(input) as ObjectResult;

            // Assert
            result.StatusCode.Should().Be(expectedResult);
        }
    }
}