using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Controllers;
using communicate_rental_backend_dev.Mappers;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.Models.DataTransfer;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using FluentAssertions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace unit_tests
{
    [TestClass]
    public class RentalObjectControllerTests
    {
        private Mock<IAllocationPeriodMapper> allocationPeriodMapperMock;
        private Mock<IAllocationPeriodRepository> allocationPeriodRepositoryMock;
        private Mock<IImageRepository> imageRepositoryMock;
        private Mock<IRentalObjectMapper> rentalObjectMapperMock;
        private Mock<IRentalObjectRepository> rentalObjectRepositoryMock;
        private RentalObjectController controller;
        private Fixture fixture;


        [TestInitialize]
        public void TestInit()
        {
            // Set up mock dependencies
            allocationPeriodMapperMock = new Mock<IAllocationPeriodMapper>();
            allocationPeriodRepositoryMock = new Mock<IAllocationPeriodRepository>();
            imageRepositoryMock = new Mock<IImageRepository>();
            rentalObjectMapperMock = new Mock<IRentalObjectMapper>();
            rentalObjectRepositoryMock = new Mock<IRentalObjectRepository>();
            controller = new RentalObjectController
            (
                // Add .Object to make them correct type for class/constructor
                allocationPeriodMapperMock.Object,
                allocationPeriodRepositoryMock.Object,
                imageRepositoryMock.Object,
                rentalObjectMapperMock.Object,
                rentalObjectRepositoryMock.Object
            );

            // Set up fixture for object creation
            fixture = new Fixture();
        }

        // NOTE: Set up functions according to Arrange - Act - Assert 
        // Look for stuff to test:
            // logical switches: if, switch, try/catch etc.
            // return values
            // Check that dependencies are called when expected -only when needed
            // Check that dependencies are called with the expected input data -state is correct at calltime


        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public async Task Get_RentalObjectIdNotFound_ShouldThrowException()
        {
            // Arrange
            var input = fixture.Create<string>();

            rentalObjectRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync((RentalObject)null);

            // Act / Assert
            await controller.Get(input);
        }

        [TestMethod]
        public async Task Get_VerifiesReturnType_ShouldReturnDTO()
        {
            // Arrange
            var input = fixture.Create<string>();

            var dbReturn = fixture.Create<RentalObject>();
            var mappedRentalObject = fixture.Create<RentalObjectDTO>();

            rentalObjectRepositoryMock
            .Setup(mock => mock.GetById(It.IsAny<string>())).ReturnsAsync(dbReturn);
            rentalObjectMapperMock
            .Setup(mock => mock.Map(It.IsAny<RentalObject>(), It.IsAny<IEnumerable<string>>())).Returns(mappedRentalObject);

            // Act
            var result = await controller.Get(input);

            // Assert
            result.Should().BeOfType<RentalObjectDTO>();
        }

        [TestMethod]
        public async Task GetRentalObjectIds_VerifiesReturnType_ShouldReturnListOfRentalObjectId()
        {
            // Arrange
            var dbResult = fixture.Create<IEnumerable<RentalObjectID>>();
            var expectedResult = fixture.Create<IEnumerable<RentalObjectID>>().GetType();

            rentalObjectRepositoryMock
            .Setup(mock => mock.GetIds()).ReturnsAsync(dbResult);

            // Act
            var result = await controller.GetRentalObjectIds();

            // Assert
            result.Should().BeOfType(expectedResult);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public async Task GetRentalObjectIds_DbReturnIsNull_ShouldThrowExcpetion()
        {
            // Arrange
            rentalObjectRepositoryMock
            .Setup(mock => mock.GetIds()).ReturnsAsync((IEnumerable<RentalObjectID>)null);

            // Act/Assert
            await controller.GetRentalObjectIds();   
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public async Task GetRentalobjects_DbReturnsNull_ShouldThrowException()
        {
            // Arrange
            rentalObjectRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync((IEnumerable<RentalObject>)null);

            // Act/Assert
            await controller.GetRentalObjects();
        }


        // METHODS THAT ARE POSSIBLE
        /*
            // [ClassInitialize]
            // public static void ClassInit(TestContext _)
            // {
            //     // Runs ONCE before any tests - try to avoid
            // }

            // [TestInitialize]
            // public void TestInit()
            // {
            //     // Runs once before every single test
            // }

            // [TestMethod]
            // public void TestMethod1()
            // {
            //     // Test
            // }

            // NOTE: Can send DataRows to TestMethods - datarow order must match parameter order
                // [DataRow(boolean, null, "someString", 123, true)]
                // [DataRow(boolean, null, "someOtherString", 0, false)]
                // public void TestMethod2(bool someBool, string someEmptyString, string someString, int SomeInt, bool shouldSucceed)
                // {
                //      // NOTE: This test is run once for each DataRow
                // }

            // [TestMethod]
            // [DataRow(null, true, "hello")] // Can have many of these for a single test
            // public void TestMethod1(object input1, bool input2, string input3)
            // {
            //     // Test
            // }

            // [TestCleanup]
            // public void TestCleanup()
            // {
            //     // Runs once after every single test - rarely necessary when not using ClassInit
            // }

            // [ClassCleanup]
            // public static void ClassCleanup()
            // {
            //     // Runs ONCE after all tests are finished - try to avoid
            // }
        */


        // FUNCTIONS THAT ARE POSSIBLE 
        // Note - also check AddRentalObject_NoPictures_ShouldCallDependencies()
        /* 
        // Create many dummy objects (x == number created)
            // var mappedAllocationPeriods = fixture.CreateMany<AllocationPeriod>(x);
        
        // Set different return values in a specific sequence - useful when created multiple objects
            // allocationPeriodMapperMock.SetupSequence(mock => mock.Map(It.IsAny<AllocationPeriodDRO>(), It.IsAny<string>()))
            //     .Returns(mappedAllocationPeriods[0])
            //     .Returns(mappedAllocationPeriods[1])
            //     .Returns(mappedAllocationPeriods[2]);

        // Async methods: Use ReturnsAsync when mocking async methods
            // rentalObjectRepositoryMock.Setup(mock => mock.GetAll()).ReturnsAsync(<something>);

        // Callback functionality lets you interact with the actual arguments sent to a mocked method
            // var actuallyMappedStuff = new List<AllocationPeriodDRO>();
            // allocationPeriodMapperMock
            //     .Setup(mock => mock.Map(It.IsAny<AllocationPeriodDRO>(), It.IsAny<string>()))
            //     .Returns(mappedAllocationPeriod)
            //     .Callback<AllocationPeriodDRO, string>((aloP, s) => actuallyMappedStuff.Add(aloP));

        // Comparison but excluding certain aspects of objects
            // var expectedResult = result;
            // result.Should().BeEquivalentTo(expectedResult, options => options.ExcludingProperties(<don't remember>));


    
        */



    }
}