using System;
using AutoFixture;
using communicate_rental_backend_dev;
using communicate_rental_backend_dev.Models.Database;
using communicate_rental_backend_dev.BusinessLogic.Services;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using unit_tests.Utilities;
using MimeKit;
using communicate_rental_backend_dev.Repositories.BlobStorage;
using communicate_rental_backend_dev.Mappers;

namespace unit_tests
{
    [TestClass]
    public class FunctionAppControllerTests
    {
        private Mock<IAllocationService> allocationServiceMock;
        private Mock<IEmailService> emailServiceMock;
        private Mock<IGraphService> graphServiceMock;
        private Mock<IAllocationPeriodRepository> allocationPeriodRepositoryMock;
        private Mock<IUserRepository> userRepositoryMock;
        private Mock<IHolidayRepository> holidayRepositoryMock;
        private Mock<IBookingRepository> bookingRepositoryMock;
        private Mock<IBlobBookingRepository> blobBookingRepositoryMock;
        private Mock<IChecklistRepository> checklistRepositoryMock;
        private Mock<IBookingMapper> bookingMapperMock;
        private FunctionAppService service;
        private Fixture fixture;

        [TestInitialize]
        public void TestInit()
        {
            // Set up Mock dependencies
            allocationServiceMock = new Mock<IAllocationService>();
            emailServiceMock = new Mock<IEmailService>();
            graphServiceMock = new Mock<IGraphService>();
            allocationPeriodRepositoryMock = new Mock<IAllocationPeriodRepository>();
            userRepositoryMock = new Mock<IUserRepository>();
            holidayRepositoryMock = new Mock<IHolidayRepository>();
            bookingRepositoryMock = new Mock<IBookingRepository>();
            blobBookingRepositoryMock = new Mock<IBlobBookingRepository>();
            checklistRepositoryMock = new Mock<IChecklistRepository>();
            bookingMapperMock = new Mock<IBookingMapper>();

            service = new FunctionAppService
            (
                allocationServiceMock.Object,
                emailServiceMock.Object,
                graphServiceMock.Object,
                allocationPeriodRepositoryMock.Object,
                userRepositoryMock.Object,
                holidayRepositoryMock.Object,
                bookingRepositoryMock.Object,
                blobBookingRepositoryMock.Object,
                checklistRepositoryMock.Object,
                bookingMapperMock.Object
            );

            fixture = new Fixture();
        }

        [TestMethod]
        public void AllocateRentals_AllocationPeriodNotNullAndDayAfterDeadline_ChecksThatExceptionIsNotThrown()
        {
            // Arrange
            var dbReturn = fixture.Create<IEnumerable<AllocationPeriod>>();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            var testObject = dbReturn.First();
            testObject.ApplicationDeadline = DateTime.Today.AddDays(-1).ToUniversalTime();

            // Act/Assert - test exception
            ObjectExtensions.DoesNotThrows<Exception>(async() => await service.AllocateRentals(DateTime.Today.Date));
        }

        [TestMethod]
        public void AllocateRentals_AllocationPeriodNotNullAnd14DaysBeforeDeadline_ChecksThatExceptionIsNotThrown()
        {
            // Arrange
            var dbReturn = fixture.Create<IEnumerable<AllocationPeriod>>();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            var testObject = dbReturn.First();
            testObject.ApplicationDeadline = DateTime.Today.AddDays(14).ToUniversalTime();

            // Act/Assert - test exception
            ObjectExtensions.DoesNotThrows<Exception>(async() => await service.AllocateRentals(DateTime.Today.Date));
        }

        [TestMethod]
        public void AllocateRentals_AllocationPeriodNullAndFirstDayOfYear_ValidatesNoException()
        {
            // Arrange
            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync((IEnumerable<AllocationPeriod>)null);
            
            // Act/Assert
            ObjectExtensions.DoesNotThrows<Exception>(async() => await service.AllocateRentals(DateTime.Today.Date));
        }

        [TestMethod]
        public void SendDeadlineReminder_AllocationPeriod_RunsWithoutExceptions()
        {
            // Arrange
            var email = fixture.Create<MailboxAddress>();
            var dbReturn = fixture.Create<IEnumerable<User>>();
            var allocationPeriod = fixture.Create<AllocationPeriod>();

            userRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            // Act/Assert
            ObjectExtensions.DoesNotThrows<Exception>(async() => await service.SendDeadlineReminders(allocationPeriod));
        }


        [TestMethod]
        public void Allocate_ALlocationPeriod_RunsWithoutExceptions()
        {
            // Arrange
            var input = fixture.Create<AllocationPeriod>();

            // Act/Asser
            ObjectExtensions.DoesNotThrows<Exception>(async () => await service.Allocate(input)); 
        }

        [TestMethod]
        public void UpdateHolidays_RunsWithoutExceptions()
        {
            // Arrange
            var newYear = new DateTime(DateTime.Today.Year, 1, 1).ToUniversalTime();

            // Act/Assert
            ObjectExtensions.
            DoesNotThrows<Exception>(async() => await service.UpdateHolidays(newYear));
        }

        [TestMethod]
        public void UpdateAllocationPeriods_NewYearDate_RunsWithoutExceptions()
        {
            // Arrange
            var newYear = new DateTime(DateTime.Today.Year, 1, 1).ToUniversalTime();

            // Act/Assert
            ObjectExtensions.
            DoesNotThrows<Exception>(async() => await service.UpdateAllocationPeriods(newYear));
        }

        [TestMethod]
        public async Task UpdateAllocationPeriods_NewYearDate_UpdatedSameNoOfTimesAsObjectNo()
        {
            // Arrange
            var input = new DateTime(DateTime.Today.Year, 1, 1).ToUniversalTime();
            var dbReturn = fixture.Create<IEnumerable<AllocationPeriod>>();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetByPredicate(It.IsAny<DateTime>())).ReturnsAsync(dbReturn);

            // Act
            await service.UpdateAllocationPeriods(input);

            // Assert
            allocationPeriodRepositoryMock
            .Verify(mock => mock.Add(It.IsAny<AllocationPeriod>()), Times.Exactly(dbReturn.Count()));
        }
        
        [TestMethod]
        public async Task UpdateAllocationPeriods_DbReturnsNull_UpdatedNullTimes()
        {
            // Arrange
            var input = new DateTime(DateTime.Today.Year, 1, 1).ToUniversalTime();

            allocationPeriodRepositoryMock
            .Setup(mock => mock.GetByPredicate(It.IsAny<DateTime>())).ReturnsAsync((IEnumerable<AllocationPeriod>)null);

            // Act
            await service.UpdateAllocationPeriods(input);

            // Assert
            allocationPeriodRepositoryMock
            .Verify(mock => mock.Add(It.IsAny<AllocationPeriod>()), Times.Exactly(0));
        }


        [TestMethod]
        public async Task UpdateUserPoints_TodayIsHiredDate_ExpectedIncreasingOfPoints()
        {
            // Arrange
            var input = DateTime.Today.ToUniversalTime();
            var dbReturn = fixture.Create<IEnumerable<User>>();
            var expectedResult = 1;

            userRepositoryMock
            .Setup(mock => mock.GetAll()).ReturnsAsync(dbReturn);

            var testObject = dbReturn.First();
            testObject.Points = 0;
            testObject.HiredDate = new DateTime(2020, DateTime.Today.Month, DateTime.Today.Day);

            // Act
            await service.UpdateUserPoints(input);

            // Assert
            testObject.Points.Equals(expectedResult);
        }
    }
}