import { Component, OnInit, Inject, OnDestroy } from '@angular/core';
import {
  MsalService,
  MsalBroadcastService,
  MSAL_GUARD_CONFIG,
  MsalGuardConfiguration,
} from '@azure/msal-angular';
import { EventMessage, EventType, InteractionType } from '@azure/msal-browser';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { Subject } from 'rxjs';
import { filter, takeUntil } from 'rxjs/operators';
import { FeedbackModalComponent } from './components/feedback-modal/feedback-modal.component';
import { MenuItems, mainMenu, adminMenu } from './models/menuItems.model';
import { RentalObjectId } from './models/rentalObject.model';
import { SingleUser } from './models/user.model';
import { RentalObjectService } from './services/rentalObject.service';
import { UserService } from './services/user.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit, OnDestroy {
  isIframe = false;
  loggedIn = false;
  mainMenu: Array<any>;
  adminMenu: Array<any>;
  rentalObjectIds: Array<RentalObjectId>;
  user: SingleUser;

  modalRef: BsModalRef;
  isMobile: boolean;
  navbarHeight = 56;
  topGap = this.navbarHeight;
  private readonly destroying$ = new Subject<void>();

  constructor(
    @Inject(MSAL_GUARD_CONFIG) private msalGuardConfig: MsalGuardConfiguration,
    private authService: MsalService,
    private msalBroadcastService: MsalBroadcastService,
    private rentalObjectService: RentalObjectService,
    private userService: UserService,
    menuItems: MenuItems,
    private modalService: BsModalService
  ) {
    // Get menu items
    menuItems.addMenu(mainMenu);
    this.mainMenu = menuItems.getMenu();
    menuItems.addMenu(adminMenu);
    this.adminMenu = menuItems.getMenu();

    // Get user
    this.getUser();

    // Create dynamic height for sidebar on scroll
    document.addEventListener('scroll', () => {
      // Check exists to make linter happy
      if (document.body.parentElement) {
        this.topGap = Math.max(0,this.navbarHeight - document.body.parentElement?.scrollTop);
      }
    });
  }

  async ngOnInit(): Promise<void> {
    // Dealing with login status through MSAL
    this.isIframe = window !== window.parent && !window.opener;
    this.checkAccount();

    // Get rentalObject Ids for navbar
    const result = await this.rentalObjectService.getAllIds();
    if (result) {
      this.rentalObjectIds = result;
    }

    this.msalBroadcastService.msalSubject$
      .pipe(
        filter(
          (msg: EventMessage) =>
            msg.eventType === EventType.LOGIN_SUCCESS ||
            msg.eventType === EventType.ACQUIRE_TOKEN_SUCCESS
        ),
        takeUntil(this.destroying$)
      )
      .subscribe(() => {
        this.checkAccount();
      });

    // Listen to resize events for navbar mode
    window.addEventListener('resize', () => {
      this.onResize();
    });

    // Decide if drawer should be open or not first time around
    this.onResize();
  }

  ngOnDestroy(): void {
    this.destroying$.next(undefined);
    this.destroying$.complete();
  }

  // **** Utility functionality ****

  openFeedbackModal() {
    this.modalRef = this.modalService.show(FeedbackModalComponent, {
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable'
    });
  }

  // Checks if breakpoint for mobile has been reached
  onResize() {
    this.isMobile = window.innerWidth < 576 ? true : false;
  }

  // **** Account related ****

  // ts-ignore is to avoid an error that shouldn't occur,
  //   ref: https://github.com/AzureAD/microsoft-authentication-library-for-js/issues/2769#issuecomment-754811027
  login() {
    if (this.msalGuardConfig.interactionType === InteractionType.Popup) {
      if (this.msalGuardConfig.authRequest) {
        this.authService
        // @ts-ignore
          .loginPopup({ ...this.msalGuardConfig.authRequest })
          .subscribe(() => this.checkAccount());
      } else {
        this.authService.loginPopup().subscribe(() => this.checkAccount());
      }
    } else {
      if (this.msalGuardConfig.authRequest) {
        // @ts-ignore
        this.authService.loginRedirect({ ...this.msalGuardConfig.authRequest });
      } else {
        this.authService.loginRedirect();
      }
    }
  }

  logout() {
    this.authService.logout();
  }

  private checkAccount() {
    this.loggedIn = this.authService.instance.getAllAccounts().length > 0;
  }

  // Gets user info
  private async getUser() {
    this.user = await this.userService.getUser();
  }
}
