import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { defineLocale } from 'ngx-bootstrap/chronos';
import { BsLocaleService } from 'ngx-bootstrap/datepicker';
import { nbLocale } from 'ngx-bootstrap/locale';
import { Booking } from 'src/app/models/booking.model';
defineLocale('nb', nbLocale);

@Component({
  selector: 'component-date-picker',
  templateUrl: './date-picker.component.html',
  styleUrls: ['./date-picker.component.scss'],
})
export class DatePickerComponent implements OnInit, OnChanges {
  @Input() disabledDays: number[]; // Array of numbers containing the days, where 0 = Sunday, 6 = Saturday
  @Input() existingBookings: Booking[]; // Array of dates
  @Output() dateChange = new EventEmitter<Date[]>();
  locale = 'nb';
  minDate = new Date();
  maxDate = new Date();
  maxLength = 14;
  disabledDates: Date[];
  dateRangepickerModel: Date[]; // Holds user input

  // ISSUE: Allows bookings crossing disabled dates
  // Probably need to put in an issue here https://github.com/valor-software/ngx-bootstrap/issues
  // Is fixed in another datepicker: https://github.com/fetrarij/ngx-daterangepicker-material/pull/270
  // Is solved by checking double dates in backend

  constructor(private localeService: BsLocaleService) {
    // Make user unable to book starting before today
    this.minDate.setDate(this.minDate.getDate());
    // Make user unable to book more than a year ahead
    this.maxDate.setDate(this.maxDate.getDate() + 366);
    // Sets locale to Norwegian
    this.localeService.use(this.locale);
  }

  ngOnInit(): void {
    this.setDisabledDates();
  }

  // Detect changes in existingBookings, to update after new booking
  ngOnChanges(changes: SimpleChanges): void {
    if (changes.existingBookings) {
      this.setDisabledDates();
      this.dateRangepickerModel = [];
    }
  }

  // Emit which dates have been selected to parent component
  onDatesSelected(): void {
    if (
      this.dateRangepickerModel !== undefined &&
      this.dateRangepickerModel.length
    ) {
      this.dateChange.emit(this.dateRangepickerModel);
    }
  }

  /**
   * Fills the array of days that should be disabled due to existing bookings
   */
  private setDisabledDates() {
    this.disabledDates = [];
    const seenDates: Date[] = [];


    // If check to avoid errors when empty
    if (this.existingBookings) {
      this.existingBookings.forEach((booking) => {

        // Creating new variable for start and end dates to get correct format for comparison
        const startDate = this.toMidday(booking.startDate);
        const endDate = this.toMidday(booking.endDate);

        // Handle overlapping dates
        this.handleBookingOverlap(seenDates, startDate);
        this.handleBookingOverlap(seenDates, endDate);

        // Creating new variable for current to keep track of which dates we add to disabled array
        // starts 1 day after start date
        const current = new Date(startDate);
        current.setDate(current.getDate() + 1);

        // Disable each day between startDate and endDate
        while (current < endDate)
        {
          this.disabledDates.push(new Date(current));
          current.setDate(current.getDate() + 1);
        }
      });
    }
  }

  /**
   * Turns dates to midday times
   *
   * @param date date to turn into midday time
   * @returns date at midday
   */
  private toMidday(date: Date): Date {
    const newDate = new Date(date);
    newDate.setHours(12, 0, 0, 0);
    return newDate;
  }

  /**
   * Adds end/start date that have aleady been seen to disabledDates to avoid open single days between bookings
   *
   * @param seenDates array of dates that's already been added
   * @param dateToCheck date to check against/add to array of seen dates
   */
  private handleBookingOverlap(seenDates: Date[], dateToCheck: Date)
  {
    if (seenDates.find(x => x.getTime() === dateToCheck.getTime())) {
      this.disabledDates.push(dateToCheck);
    }
    else {
      seenDates.push(dateToCheck);
    }
  }
}
