import { RentalObject } from 'src/app/models/rentalObject.model';
import { Component, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { Map, Feature } from 'ol';
import * as Controls from 'ol/control';
import * as Source from 'ol/source';
import * as Layer from 'ol/layer';
import * as Style from 'ol/style';
import Point from 'ol/geom/Point';
import XYZ from 'ol/source/XYZ';
import { fromLonLat } from 'ol/proj';
import { NavigateControl } from './navigateControl';
//import { toLonLat } from 'ol/proj';
//import { toStringHDMS } from 'ol/coordinate';
//import Overlay from 'ol/Overlay';

// TODO: Swap to FontAwesome marker
// https://fontawesome.com/icons/map-marker-alt?style=solid

@Component({
  selector: 'component-map',
  templateUrl: './map.component.html',
  styleUrls: ['./map.component.scss'],
})
export class MapComponent implements OnInit, OnChanges {
  @Input() rentalObjects: Array<RentalObject>;
  map: any;
  // Map center position
  centerLatitude = 50;
  centerLongitude = 6;
  // Marker style and source
  markerSource: Source.Vector;
  markerStyle = new Style.Style({
    image: new Style.Icon({
      crossOrigin: 'anonymous',
      src: 'assets/map-marker.png',
      opacity: 1,
      scale: 0.08,
      anchor: [255, 1],
      // @ts-ignore
      anchorXUnits: 'fractions',
      anchorYunits: 'fractions'
    }),
  });

  constructor() {}

  ngOnInit(): void {
    this.initialiseMap();
    /* This stuff doesn't work, but probably a good place to start for marker popup
    ref: https://openlayers.org/en/latest/examples/icon.html
    var element = document.getElementById('popup');
    var popup = new Overlay({
      // @ts-ignore
      element: element,
      // @ts-ignore
      positioning: 'botom-center',
      stopEvent: false,
      offset: [0, -50]
    });
    this.map.addOverlay(popup);

    element?.addEventListener('mousemove', e => {
      this.changePointer(e);
    })
    element?.addEventListener('click', e => {
      console.log('click');
      this.checkLocation(e);
    })
    */
  }

  initialiseMap() {
    this.markerSource = new Source.Vector();
    // Add map marker for each rentalObject
    this.rentalObjects.forEach((object: RentalObject) => {
      this.addMarker(object.longtitude, object.latitude, object.title);
    });

    // Set up map view
    this.map = new Map({
      target: 'map',
      layers: [
        // Tile layer: the layer displaying the map
        new Layer.Tile({
          source: new XYZ({
            url: 'https://{a-c}.tile.openstreetmap.org/{z}/{x}/{y}.png',
          }),
        }),

        // Vector layer: the layer displaying markers
        new Layer.Vector({
          source: this.markerSource,
          style: this.markerStyle,
        }),
      ],

      // Turn off controls
      controls: Controls.defaults({
        zoom: false,
        attribution: false,
        rotate: false,
      }),
    });

    if (this.rentalObjects.length === 1) {
      this.map.controls.extend([new NavigateControl(this.rentalObjects[0].latitude, this.rentalObjects[0].longtitude)]);
    }

    // Zoom to correct place
    this.adjustZoom();

    // Listen to resize events for navbar mode
    window.addEventListener('resize', () => {
      this.adjustZoom();
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    // Check that change is on rentalObjects, and that it's not the first change
    if (changes.rentalObjects && !changes.rentalObjects.firstChange) {
      // Set current map target to undefined to remove old map
      this.map.setTarget(undefined);
      // Then delete old map object to avoid endless amounts in memory
      this.map = null;
      // Then initialize it again
      this.initialiseMap();
    }
  }

  // Add marker to map
  private addMarker(lon: number, lat: number, name: string) {
    const iconFeature = new Feature({
      geometry: new Point(fromLonLat([lon, lat])),
      name,
    });

    this.markerSource.addFeature(iconFeature);
  }

  private adjustZoom() {
    this.map.getView().fit(this.markerSource.getExtent(), {padding: [50, 0, 50, 0], minResolution: 60});
  }

  /*
  checkLocation(evt: any) {
    console.log('click');
    var feature = this.map.forEachFeatureAtPixel(evt.pixel);
    console.log(feature);

    if (feature) {
      var coordinates = feature.getGeometry().getCoordinates();
      this.map.popup.setPosition(coordinates);
      this.map.popover({
        placement: 'top',
        html: true,
        content: feature.get('name'),
      });
      this.map.popover('show');
    } else {
      this.map.popover('dispose');
    }
  }

  changePointer(evt: any) {
    console.log('mousemove');
    if (evt.dragging) {
      this.map.popover('dispose');
    }
    var pixel = this.map.getEventPixel(evt.originalEvent);
    var hit = this.map.hasFeatureAtPixel(pixel);
    this.map.getTarget().style.cursor = hit ? 'pointer' : '';
  }
  */
}
