import { Component, Input, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { Booking } from 'src/app/models/booking.model';
import { CalendarOptions } from '@fullcalendar/angular';
import bootstrapPlugin from '@fullcalendar/bootstrap';

@Component({
  selector: 'component-overviewcalendar',
  templateUrl: './overviewcalendar.component.html',
  styleUrls: ['./overviewcalendar.component.scss']
})
export class OverviewcalendarComponent implements OnInit, OnChanges {
  @Input() bookings: Booking[];
  tooltipBox: HTMLElement;
  calendarOptions: CalendarOptions = {
    initialView: 'dayGridMonth',
    events: [],
    locale: 'nb',
    firstDay: 1,
    plugins: [ bootstrapPlugin ],
    themeSystem: 'bootstrap',
    eventColor: '#9c0a0a',
    contentHeight: 'auto',
    buttonText:
    {
      today:    'nå',
      month:    'måned',
      week:     'uke',
      day:      'dag',
      list:     'liste'
    },
    eventMouseEnter: this.onHover.bind(this),
    eventMouseLeave: this.exitHover.bind(this)
  };

  constructor() { }

  ngOnInit(): void {
    this.addEventsToCalendar();
    // Can't make this error go away even if null checking, so ignoring the no-non-null-assertion
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    this.tooltipBox = document.getElementById('tooltipBox')!;
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.bookings && !changes.bookings.firstChange) {
      this.addEventsToCalendar();
    }
  }

  onHover(input: any) {
    this.tooltipBox.innerHTML = input.event.title;
    this.tooltipBox.style.visibility = 'visible';
    this.tooltipBox.style.top = (input.jsEvent.clientY - 35) + 'px';
    this.tooltipBox.style.left = (input.jsEvent.clientX - 60) + 'px';
  }

  exitHover(input: any) {
    this.tooltipBox.style.visibility = 'hidden';
  }


  private addEventsToCalendar() {
    const events: EventInput[] = [];

    const uniqueTitles = [...new Set(this.bookings.map(booking => booking.englishTitle).sort())];
    const colorlist = [
      '#9c0a0a', // communicate red
      '#038773', // mid green
      '#f26835', // dark yellow
      '#424040', // medium-dark gray
      '#F2A413', // orange
      '#1c4740'  // dark green
    ];


    const colorMap = new Map<string, string>();
    uniqueTitles.forEach((title: string , i: number) =>
      // Mapping with modulus to make list start at beginning when more rental objects
      colorMap.set(title, colorlist[i % colorlist.length]));

    this.bookings.forEach(booking => {
      const endDate = new Date(booking.endDate);
      endDate.setDate(endDate.getDate() + 1);

      events.push(new EventInput(
        booking.title + ': ' + booking.userName,
        booking.startDate,
        endDate,
        colorMap.get(booking.englishTitle) ?? '#9c0a0a'
      ));
    });
    this.calendarOptions.events = events;
  }
}

export class EventInput {
  public title: string;
  public start: Date;
  public end: Date;
  public backgroundColor: string;
  public borderColor: string;

  constructor(
    title: string,
    start: Date,
    end: Date,
    colour: string
  ) {
    this.title = title;
    this.start = start;
    this.end = end;
    this.backgroundColor = colour;
    this.borderColor = colour;
  }
}
