import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Booking } from '../models/booking.model';
import { HttpService } from './http.service';

@Injectable()
export class BookingService {
  constructor(private httpService: HttpService) {}

  /**
   * Retrieves all bookings from the backend
   *
   * @returns Observable<Booking[]>
   */
  getAll(): Observable<Booking[]> {
    return this.httpService.get<Booking[]>('Bookings/');
  }

  /**
   * Retrieves all bookings for the specified user
   *
   * @param userId
   * @returns Observable<Booking[]>
   */
  getByUser(userId: string): Observable<Booking[]> {
    return this.httpService.get<Booking[]>('Bookings/users/' + userId);
  }


  /**
   * Retrieves all confirmed bookings for the specified location
   *
   * @param locationEnglishTitle - the rental object's english title
   * @returns Observable<Booking[]>
   */
  getConfirmedByRentalObject(
    locationEnglishTitle: string
  ): Observable<Booking[]> {
    return this.httpService.get<Booking[]>(
      'Bookings/locations/' + locationEnglishTitle
    );
  }

  postComment(booking: Booking){
    console.log(booking);
    return this.httpService.update('Bookings', booking);
  }

  /**
   * Creates new booking or booking application.
   * Booking property needs the following properties:
   *
   * @param booking - booking object
   * @returns Observable<boolean> - true if complete, false if error.
   */
  createNew(booking: Booking) {
    return this.httpService.postObserveResponse('Bookings', booking);
  }

  /**
   * Sends cancellation for a booking.
   *
   * @param id - ID for the booking to be cancelled.
   * @param locationEnglishTitle - the rental object's english title (id).
   * @returns Observable<boolean> - true if complete, false if error.
   */
  cancel(id: string, locationEnglishTitle: string): Observable<boolean> {
    return this.httpService.delete<boolean>(
      'Bookings/locations/',
      id,
      locationEnglishTitle
    );
  }
}
