import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { Checklist, ChecklistItem, CompletedChecklist } from '../models/checklist.model';
import { HttpService } from './http.service';

@Injectable({
  providedIn: 'root',
})
export class ChecklistService {
  constructor(private httpService: HttpService) {}

  /**
   * Sends completed checklist to backend.
   *
   * @param userId - the user ID that completed the checklist.
   * @param bookingId - the booking ID the checklist is for.
   * @param locationTitle - the rental object's title ('name').
   * @param startDate - the start date of the booking the checklist is for.
   * @param endDate - the end date of the booking the checklist is for.
   * @param checkedItems - ChecklistItem[] containing string of completed items and true/false if checked.
   * @param comment - User comment on the rental object.
   * @returns Observable<boolean> - true if complete, false if error.
   */
  send(
    userId: string,
    bookingId: string,
    locationTitle: string,
    startDate: Date,
    endDate: Date,
    checkedItems: ChecklistItem[],
    comment: string,
  ): Observable<boolean> {
    const checklist = new Checklist(
      userId,
      bookingId,
      locationTitle,
      startDate,
      endDate,
      checkedItems,
      comment
    );
    return this.httpService.post<boolean>('Checklists', checklist);
  }

  getExisting() {
    return this.httpService.get<CompletedChecklist[]>('Checklists');
  }


  /**
   * Gets checklist array for a specific rental object.
   *
   * @param englishTitle - rental object id to get object by.
   * @returns Observable<string[]>
   */
   getChecklist(englishTitle: string, bookingId: string): Observable<CompletedChecklist> {
    return this.httpService.get<CompletedChecklist>(
      'Checklists/' + englishTitle + '/' + bookingId
    );
  }

}
