import {
  HttpClient,
  HttpErrorResponse,
  HttpHeaders,
} from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { environment } from 'src/environments/environment';

@Injectable()
export class HttpService {

    httpOptions = {
        headers: new HttpHeaders({
        'ocp-apim-subscription-key': environment.apimKey,
        'content-type': 'application/json',
        }),
        withCredentials: true
    };

  constructor(private http: HttpClient) {}

  /**
   * Get REST call for methods not aimed at backend.
   *
   * @param url - full url the get request is sent to.
   * @returns Observable<T> of expected return type.
   */
  public getFromPath<T>(url: string): Observable<T> {
    return this.http.get<T>(`${url}`).pipe(catchError(this.handleError));
  }

  // Get all if no parameters, otherwise can be used as ie. getById
  /**
   * Get REST call to backend. Parameters are voluntary.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param parameters
   * @returns Observable<T> of expected return type.
   */
  public get<T>(path: string, parameters?: string): Observable<T> {
    let fullPath = path;
    if (parameters) {
      fullPath += parameters;
    }
    return this.http
      .get<T>(`${environment.apiUrl}${fullPath}`, this.httpOptions)
      .pipe(catchError(this.handleError));
  }

  /**
   * Post REST call to backend.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param body - REST body.
   * @returns Observable<T> of expected return type.
   */
  public post<T>(path: string, body: any): Observable<T> {
    return this.http
      .post<T>(`${environment.apiUrl}${path}`, body, this.httpOptions)
      .pipe(catchError(this.handleError));
  }

  /**
   * Post REST call to backend. Made for calls where observing the response is necessary.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param body - REST body.
   * @returns Observable<T> of expected return type.
   */
  public postObserveResponse(path: string, body: any) {
    return this.http
      .post(`${environment.apiUrl}${path}`, body, {
        headers: new HttpHeaders({
          'ocp-apim-subscription-key': environment.apimKey,
          'content-type': 'application/json',
          }),
        withCredentials: true,
        observe: 'response'
      });
  }

  /**
   * Post REST call to backend. Made for calls where posting FormData is necessary.
   * This call observes the entire response, not just the body.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param body - REST body.
   * @returns Observable<T> of expected return type.
   */
  public postFormData(path: string, body: any) {
    return this.http
      .post(`${environment.apiUrl}${path}`, body, {
        headers: new HttpHeaders({
          'ocp-apim-subscription-key': environment.apimKey
          }),
        withCredentials: true,
        observe: 'response'
      });
  }

  /**
   * Put REST call to backend.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param body - REST body.
   * @returns Observable<T> of expected return type.
   */
  public update<T>(path: string, body: any): Observable<T> {
    return this.http
      .put<T>(`${environment.apiUrl}${path}`, body, this.httpOptions)
      .pipe(catchError(this.handleError));
  }

  /**
   * Put REST call to backend. Made for calls posting FormData.
   * The call observes the entire response, not just the body.
   *
   * @param path - URI fragment (ie. "Bookings/") for the REST call.
   * @param body - REST body.
   * @returns Observable<T> of expected return type.
   */
  public updateFormData(path: string, body: any) {
  return this.http
    .put(`${environment.apiUrl}${path}`, body, {
      headers: new HttpHeaders({
        'ocp-apim-subscription-key': environment.apimKey
        }),
      withCredentials: true,
      observe: 'response'
    });
   }

  /**
   * Delete REST call to backend. Subquery is voluntary.
   *
   * @param path
   * @param id
   * @param subquery
   * @returns
   */
  public delete<T>(path: string, id: string, subquery?: string): Observable<T> {
    if (subquery) {
      subquery += '/';
    } else {
      subquery = '';
    }
    return this.http
      .delete<T>(
        `${environment.apiUrl}${path}${subquery}${id}`,
        this.httpOptions
      )
      .pipe(catchError(this.handleError));
  }

  // Handles all errors.
  private handleError(error: HttpErrorResponse) {
    let errorMessage = '';
    if (error.error instanceof ErrorEvent) {
      errorMessage = error.error.toString();
    } else {
      errorMessage = `Error code: ${error.status}\nMessage: ${error.error}`;
    }
    console.log(errorMessage);
    return throwError(errorMessage);
  }
}
