import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import {
  RentalObject,
  RentalObjectId,
  SendData,
} from '../models/rentalObject.model';
import { HttpService } from './http.service';

@Injectable()
export class RentalObjectService {
  rentalObjects: RentalObject[] = [];

  constructor(private httpService: HttpService) {}

  /**
   * Get all rental objects.
   *
   * @returns RentalObject[]
   */
  async getAll() {
    let rentalObjects = this.rentalObjects;
    if (rentalObjects.length === 0) {
      await this.httpService.get<RentalObject[]>('RentalObject')
      .toPromise()
      .then(result => {
        this.rentalObjects = result;
        rentalObjects = this.rentalObjects;
      });
    }

    return rentalObjects;
  }

  /**
   * Get all rental object ids (englishTitle) and titles.
   *
   * @returns RentalObjectId[]
   */
  async getAllIds(): Promise<RentalObjectId[]> {
    if (this.rentalObjects.length === 0) {
      await this.getAll();
    }

    return this.rentalObjects as RentalObjectId[];
  }

  /**
   * Get specific rental object.
   *
   * @param englishTitle - id to get object by.
   * @returns RentalObject
   */
  async getByEnglishTitle(englishTitle: string) {
    let rentalObject = this.rentalObjects.find(x => x.englishTitle === englishTitle);

    if (!rentalObject) {
      await this.httpService.get<RentalObject>('RentalObject/' + englishTitle)
      .toPromise()
      .then(result => {
        rentalObject = result;
        this.getAll();
      });
    };

    return rentalObject;
  }

  /**
   * Creates a new rental object.
   *
   * @param rentalObject - full rental object to create.
   * @returns Observable<boolean> - true if complete, false if error.
   */
  create(rentalObject: FormData) {
    return this.httpService.postFormData('RentalObject', rentalObject);
  }

  /**
   * Updates existing rental object.
   *
   * @param rentalObject - full rental object info.
   * @param rentalObjectEnglishTitle - id of rental object to update
   * @returns Observable<boolean> - true if complete, false if error.
   */
  update(rentalObject: FormData) {
    return this.httpService.updateFormData('RentalObject', rentalObject);
  }

  /**
   * Removes existing rental object.
   *
   * @param rentalObjectEnglishTitle - id of rental object to delete.
   * @returns Observable<boolean> - true if complete, false if error.
   */
  remove(rentalObjectEnglishTitle: string) {
    return this.httpService.delete(
      'RentalObject/',
      rentalObjectEnglishTitle
    );
  }
}
