import { Injectable } from '@angular/core';
import { SingleUser, SingleDbUser, GraphUser, NewUser } from '../models/user.model';
import { HttpService } from './http.service';
import { Observable } from 'rxjs';


@Injectable()
export class UserService {
  userInfo: SingleUser = new SingleUser();
  private readonly graphEndpoint = 'https://graph.microsoft.com/v1.0/me';

  constructor(private httpService: HttpService) {}

  /**
   * Get full user object, including info from MSGraph and database.
   *
   * @returns Promise<User>
   */
  public getAllUsers(): Observable<NewUser[]> {
    return this.httpService.get<NewUser[]>('User/');
  }
  /**
   * Get full user object, including info from MSGraph and database.
   *
   * @returns Promise<User>
   */
  public async getUser() {
    if (this.userInfo === undefined || this.userInfo.points == null) {
      await this.getUserDbInfo();
    }
    return this.userInfo;
  }

  /**
   * Get user's ID
   *
   * @returns Promise<userId: string>
   */
  public async getUserId() {
    if (this.userInfo === undefined || this.userInfo.id == null) {
      await this.getUserGraphInfo();
    }
    return this.userInfo.id;
  }

  /**
   * Get user's name
   *
   * @returns Promise<name: string>
   */
  public async getUserName() {
    if (this.userInfo === undefined || this.userInfo.name == null) {
      await this.getUserGraphInfo();
    }

    return this.userInfo.name;
  }

  /**
   * get user's current points
   *
   * @returns Promise<points: number>
   */
  public async getUserPoints() {
    if (this.userInfo === undefined || this.userInfo.points == null) {
      await this.getUserDbInfo();
    }

    return this.userInfo.points;
  }

  /**
   * Add user to database
   *
   * @param user DbUser containing hire date, points and user id
   * @returns Promise<boolean> - true if complete, false if error.
   */
  public add(user: NewUser) {
    return this.httpService.post<boolean>('User', user);
  }

   /**
    * Add user to database
    *
    * @param user DbUser containing hire date, points and user id
    * @returns Promise<boolean> - true if complete, false if error.
    */
    public update(user: NewUser) {
      return this.httpService.post<boolean>('User', user);
    }

  /**
   * Removes existing user
   *
   * @param user DbUser containing hire date, points, user email and ifAdmin
   * @returns Promise<boolean> - true if complete, false if error.
   */
  public delete(email: string) {
    return this.httpService.delete<boolean>('User/', email);
  }

  // Gets user info from MSGraph
  private async getUserGraphInfo() {
    // Get userinfo from msgraph
    const result = await this.httpService
      .getFromPath<GraphUser>(this.graphEndpoint)
      .toPromise();
    this.userInfo.id = result.id;
    this.userInfo.name = result.displayName;
  }

  // Get user info from database
  private async getUserDbInfo() {
    // Get userinfo from DB - also calls getUserId and as such also gets msgraph info
    await this.httpService
      .get<SingleDbUser>('User/' + (await this.getUserId()))
      .toPromise()
      .then(result => {
        this.userInfo.points = result.points;
        this.userInfo.isAdmin = result.isAdmin;
      })
      // Set 0 points if not exists.
      .catch(() => (this.userInfo.points = 0));
  }
}
