import { Component, OnInit } from '@angular/core';
import { AlertComponent } from 'ngx-bootstrap/alert';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { CancellationModalComponent } from 'src/app/components/cancellation-modal/cancellation-modal.component';
import { Booking } from 'src/app/models/booking.model';
import { BookingService } from 'src/app/services/booking.service';
import { UserService } from 'src/app/services/user.service';
import { ChecklistModalComponent } from '../../components/checklist-modal/checklist-modal.component';

@Component({
  selector: 'app-booking-overview',
  templateUrl: './booking-overview.component.html',
  styleUrls: ['./booking-overview.component.scss'],
})
export class BookingOverviewComponent implements OnInit {
  isLoaded = false;
  alerts: any[] = [];
  bookings: Booking[];
  today: Date;
  private modalRef: BsModalRef;
  private userId: string;

  constructor(
    private bookingService: BookingService,
    private userService: UserService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getBookings();

    this.today = new Date();
  }

  confimCancellation(booking: Booking): void {
    // Create cancellation modal, passing relevant booking
    const initialState = { booking };
    this.modalRef = this.modalService.show(CancellationModalComponent, {
      initialState,
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });

    // Listen to event from cancellation modal
    this.modalRef.content.cancellationCallComplete.subscribe(
      (completed: boolean) => {
        if (completed) {
          this.getBookings();
          this.alerts.push({
            type: 'success',
            msg: 'Bookingen har blitt kansellert.',
            timeout: 10000
          });
        } else {
          this.alerts.push({
            type: 'danger',
            msg: 'Kunne ikke oppdatere booking - prøv igjen.',
            timeout: 10000
          });
        }
      }
    );
  }

  openChecklist(booking: Booking): void {
    // Create checklist modal, passing relevant booking
    const initialState = { booking };
    this.modalRef = this.modalService.show(ChecklistModalComponent, {
      initialState,
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });

    this.modalRef.content.checklistComplete.subscribe((completed: boolean) => {
      if (completed) {
        this.alerts.push({
          type: 'success',
          msg: 'Sjekklisten er sendt!',
          timeout: 10000
        });
      } else {
        this.alerts.push({
          type: 'danger',
          msg: 'Kunne ikke sende sjekkliste - prøv igjen.',
          timeout: 10000
        });
      }
    });
  }

  onClosedAlert(dismissedAlert: AlertComponent): void {
    this.alerts = this.alerts.filter(alert => alert !== dismissedAlert);
  }

  private async getBookings() {
    this.bookingService
      .getByUser(await this.getUserId())
      .subscribe(results => {
        this.bookings = results;
        this.bookings.forEach((booking) => {
          // Set date format for comparison in ngIf in template
          booking.startDate = new Date(booking.startDate);
        });
        this.bookings = this.bookings.sort((a, b) => b.startDate.getTime() - a.startDate.getTime());
        this.isLoaded = true;
      }, error => {
        this.bookings = [];
        this.isLoaded = true;
      });
  }

  private async getUserId(): Promise<string> {
    if (!this.userId) {
      this.userId = await this.userService.getUserId();
    }

    return this.userId;
  }
}
