import { Component, OnInit, TemplateRef, OnDestroy } from '@angular/core';
import {
  trigger,
  transition,
  style,
  animate
} from '@angular/animations';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { Booking } from 'src/app/models/booking.model';
import {
  RentalObject,
  PaymentInfo,
  PAYMENT_INFO,
} from 'src/app/models/rentalObject.model';
import { RentalObjectService } from 'src/app/services/rentalObject.service';
import { BookingService } from 'src/app/services/booking.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Subscription } from 'rxjs/internal/Subscription';
import { UserService } from 'src/app/services/user.service';
import { SingleUser } from 'src/app/models/user.model';
import { AlertComponent } from 'ngx-bootstrap/alert';

@Component({
  selector: 'app-rental-object',
  animations: [
    trigger('slideIn', [
      transition(':enter', [
        style({ transform: 'translateY(100%)', opacity: 0 }),
        animate('150ms', style({ transform: 'translateX(0)', opacity: 1 })),
      ]),
    ]),
  ],
  templateUrl: './rental-object.component.html',
  styleUrls: ['./rental-object.component.scss'],
})
export class RentalObjectComponent implements OnInit, OnDestroy {
  rentalObject: RentalObject;
  isLoaded = false;
  bookingsIsLoaded = false;

  activeSlideIndex = 0;
  btnIsDisabled = true;
  directionsIsCollapsed = true;
  equipmentIsCollapsed = true;
  fireIsCollapsed = true;
  otherPracticalInfoIsCollapsed = true;
  rentalInfoIsCollapsed = true;
  rulesIsCollapsed = false;
  slides: { image: string }[];
  mapRentalObjectArray: RentalObject[];


  bookings: Booking[];
  alerts: any[] = [];
  paymentInfo: PaymentInfo;
  selectedDates: Date[];
  modalRef: BsModalRef;
  user: SingleUser;
  private routeSub: Subscription;

  constructor(
    private bookingService: BookingService,
    private rentalObjectService: RentalObjectService,
    private userService: UserService,
    private modalService: BsModalService,
    private route: ActivatedRoute,
    private router: Router
  ) {
    this.paymentInfo = PAYMENT_INFO;
    this.getUser();
  }

  async ngOnInit(): Promise<void> {
    // Get rentalobject id from path
    this.routeSub = this.route.params.subscribe(async (params) => {
      // Get rentalobject and check that exists
      const result = await this.rentalObjectService.getByEnglishTitle(params.id);
      if (result) {
        this.rentalObject = result;

        // Push new rental object to map
        this.mapRentalObjectArray = [];
        this.mapRentalObjectArray.push(this.rentalObject);

        // Push images to slides
        this.initialiseCarousel();
        await this.getBookings();

        // Set loaded to true to display
        this.isLoaded = true;

      } else {
        this.router.navigate(['404']);
      }
    });

  }

  ngOnDestroy(): void {
    // Removes subscription to route
    if (this.routeSub) {
      this.routeSub.unsubscribe();
    }
  }

  handleDateSelection(selectedDates: Date[]): void {
    // If dates have been selected -> enable button
    if (!selectedDates.length) {
      this.btnIsDisabled = true;
    } else {
      this.btnIsDisabled = false;
      this.selectedDates = selectedDates;
    }
  }

  // Confirmation dialogue functions
  openModal(confirmationDialogue: TemplateRef<any>): void {
    this.modalRef = this.modalService.show(confirmationDialogue, {
      class: 'modal-lg modal-dialog-centered',
    });
  }

  async confirmBooking() {
    this.modalRef.hide();

    // If rental object has a standard checkin/out time, set booking to that time
    if (this.rentalObject.rentalPeriodsInnTime && this.rentalObject.rentalPeriodsOutTime) {
      // Must ignore errors because the error checker don't know what hours/minute properties are, even though they are present
      // @ts-ignore
      this.selectedDates[0].setHours(this.rentalObject.rentalPeriodsInnTime.hours, this.rentalObject.rentalPeriodsInnTime.minutes, 0);
      // @ts-ignore
      this.selectedDates[1].setHours(this.rentalObject.rentalPeriodsOutTime.hours, this.rentalObject.rentalPeriodsOutTime.minutes, 0);
    }

    // Create new booking object
    const booking = new Booking(
      await this.userService.getUserId(),
      this.rentalObject.englishTitle,
      this.rentalObject.title,
      this.selectedDates[0],
      this.selectedDates[1]
    );

    // Send booking to backend
    this.bookingService
      .createNew(booking)
      .subscribe(result => {
        // Give alert about successful booking or booking application
        if (result.status === 201) {
          this.alerts.push({
            type: 'success',
            msg: 'Bookingen er lagt inn!',
            timeout: 10000
          });
          // Create new array to ensure change is detected in child date picker
          this.bookings = [...this.bookings, booking];
        } else if (result.status === 202) {
          this.alerts.push({
            type: 'success',
            msg: 'Søknad om booking er lagt inn.',
            timeout: 10000
          });
        }
      }, error => {
        // Give message about error in booking with conflict or other
        console.log(error);
        if (error.status === 409) {
          this.alerts.push({
            type: 'danger',
            msg: 'Kunne ikke legge in booking - ' + error.error +'.',
            timeout: 10000
          });
        } else {
          this.alerts.push({
            type: 'danger',
            msg: 'Noe gikk galt - prøv igjen.',
            timeout: 10000
          });
        }
      });
  }

  onClosedAlert(dismissedAlert: AlertComponent): void {
    this.alerts = this.alerts.filter(alert => alert !== dismissedAlert);
  }

  scroll(el: HTMLElement) {
    el.scrollIntoView();
  }

  open(modal: TemplateRef<any>) {
    this.modalRef = this.modalService.show(modal , {
      class: 'modal-lg modal-dialog-centered modal-dialog-scrollable',
    });
  }

  confirmRemoval() {
    this.modalRef.hide();
    // Remove user
    this.rentalObjectService.remove(this.rentalObject.englishTitle).subscribe(
      () => {
        window.location.href = '/';
      }
    );
  }

  private async getBookings() {
    this.bookings = [];
    this.bookings = await this.bookingService
      .getConfirmedByRentalObject(this.rentalObject.englishTitle)
      .toPromise();
    this.bookingsIsLoaded = true;
  }

  // Push images to carousel array
  private initialiseCarousel(): void {
    this.slides = [];
    this.rentalObject.pictures.forEach((picture) => {
      this.slides.push({
        image: picture,
      });
    });
    // Reset active slide to 0 to avoid errors
    this.activeSlideIndex = 0;
  }

   // Gets user info
   private async getUser() {
    this.user = await this.userService.getUser();
  }

}

